@@ -1,5 +1,5 @@
 /* 
- * $Id: ftp_session.c,v 1.48 2008/12/23 09:48:21 d1mag Exp $
+ * $Id: ftp_session.c,v 1.67 2012/03/13 00:06:35 alfille Exp $
  */
 
 #include "owftpd.h"
@@ -13,7 +13,7 @@
 /* prototypes */
 static int invariant(const struct ftp_session_s *f);
 static void reply(struct ftp_session_s *f, int code, const char *fmt, ...);
-static void change_dir(struct ftp_session_s *f, char *new_dir);
+static void change_dir(struct ftp_session_s *f, const char *new_dir);
 static int open_connection(struct ftp_session_s *f);
 static int write_fully(int file_descriptor, const char *buf, int buflen);
 static void init_passive_port(void);
@@ -324,7 +324,7 @@ static void reply(struct ftp_session_s *f, int code, const char *fmt, ...)
 	va_end(ap);
 
 	/* log our reply */
-	LEVEL_DATA("%s %s\n", f->client_addr_str, buf);
+	LEVEL_DATA("%s %s", f->client_addr_str, buf);
 
 	/* send the output to the other side */
 	telnet_session_println(f->telnet_session, buf);
@@ -411,7 +411,7 @@ static void get_addr_str(const sockaddr_storage_t * s, char *buf, int bufsiz)
 
 static void do_cwd(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 {
-	char *new_dir;
+	const char *new_dir;
 
 	daemon_assert(invariant(f));
 	daemon_assert(cmd != NULL);
@@ -434,15 +434,15 @@ static void do_cdup(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 	daemon_assert(invariant(f));
 }
 
-static void change_dir(struct ftp_session_s *f, char *new_dir)
+static void change_dir(struct ftp_session_s *f, const char *new_dir)
 {
 	struct cd_parse_s cps;
 
 	strcpy(cps.buffer, (ASCII *) f->dir);
-	cps.rest = new_dir;
+	strcpy(cps.rest, new_dir);
 	cps.pse = parse_status_init;
 
-	LEVEL_DEBUG("CD dir=%s, file=%s\n", SAFESTRING(cps.buffer), SAFESTRING(new_dir));
+	LEVEL_DEBUG("CD dir=%s, file=%s", SAFESTRING(cps.buffer), SAFESTRING(new_dir));
 	FileLexCD(&cps);
 
 	switch (cps.solutions) {
@@ -453,8 +453,9 @@ static void change_dir(struct ftp_session_s *f, char *new_dir)
 	default:
 		cps.ret = -EFAULT;
 	}
-	if (cps.dir == NULL)
+	if (cps.dir == NULL) {
 		cps.ret = -ENOMEM;
+	}
 
 	if (cps.ret) {
 		reply(f, 550, "Error changing directory. %s", strerror(-cps.ret));
@@ -463,8 +464,12 @@ static void change_dir(struct ftp_session_s *f, char *new_dir)
 		reply(f, 250, "Directory change successful.");
 	}
 
-	if (cps.dir)
+	if (cps.dir) {
 		free(cps.dir);
+	}
+	if (cps.rest) {
+		free(cps.dir);
+	}
 }
 
 static void do_quit(struct ftp_session_s *f, const struct ftp_command_s *cmd)
@@ -582,6 +587,7 @@ static int set_pasv(struct ftp_session_s *f, sockaddr_storage_t * bind_addr)
 		reply(f, 500, "Error creating server socket; %s.", strerror(errno));
 		return -1;
 	}
+	fcntl (socket_fd, F_SETFD, FD_CLOEXEC); // for safe forking
 
 	for (;;) {
 		port = get_passive_port();
@@ -779,9 +785,9 @@ static int get_passive_port()
 	pthread_once(&once_control, init_passive_port);
 
 	/* pick a random port between 1024 and 65535, inclusive */
-	pthread_mutex_lock(&mutex);
+	_MUTEX_LOCK(mutex);
 	port = (lrand48() % 64512) + 1024;
-	pthread_mutex_unlock(&mutex);
+	_MUTEX_UNLOCK(mutex);
 
 	return port;
 }
@@ -912,9 +918,8 @@ static void do_retr(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 	struct timeval transfer_time;
 	struct one_wire_query owq;
 	size_t size_write;
-	int returned_length;
+	SIZE_OR_ERROR returned_length;
 	off_t offset = 0;
-	int need_owq_destroy = 1;
 
 	daemon_assert(invariant(f));
 	daemon_assert(cmd != NULL);
@@ -929,13 +934,13 @@ static void do_retr(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 
 	/* if the last command was a REST command, restart at the */
 	/* requested position in the file                         */
-	if ((f->file_offset_command_number == (f->command_number - 1)))
+	if ((f->file_offset_command_number == (f->command_number - 1))) {
 		offset = f->file_offset;
+	}
 
 	/* Can we parse the name? */
-	if (FS_OWQ_create_plus(f->dir, file_name, NULL, 0, offset, &owq)) {
+	if ( BAD( OWQ_create_plus(f->dir, file_name, &owq) ) ) {
 		reply(f, 550, "File does not exist.");
-		need_owq_destroy = 0;
 		goto exit_retr;
 	}
 
@@ -955,12 +960,11 @@ static void do_retr(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 		goto exit_retr;
 	}
 
-	OWQ_size(&owq) = FullFileLength(PN(&owq));
-	OWQ_buffer(&owq) = malloc(OWQ_size(&owq) - offset);
-	if (OWQ_buffer(&owq) == NULL) {
+	if ( BAD( OWQ_allocate_read_buffer(&owq)) ) {
 		reply(f, 550, "Error, file too large.");
 		goto exit_retr;
 	}
+	OWQ_offset(&owq) = offset ;
 
 	returned_length = FS_read_postparse(&owq);
 	if (returned_length < 0) {
@@ -1014,28 +1018,17 @@ static void do_retr(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 	gettimeofday(&end_timestamp, NULL);
 
 	/* calculate transfer rate */
-	transfer_time.tv_sec = end_timestamp.tv_sec - start_timestamp.tv_sec;
-	transfer_time.tv_usec = end_timestamp.tv_usec - start_timestamp.tv_usec;
-	while (transfer_time.tv_usec >= 1000000) {
-		transfer_time.tv_sec++;
-		transfer_time.tv_usec -= 1000000;
-	}
-	while (transfer_time.tv_usec < 0) {
-		transfer_time.tv_sec--;
-		transfer_time.tv_usec += 1000000;
-	}
+	timersub( &end_timestamp, &start_timestamp, &transfer_time ) ;
 
 	/* note the transfer */
-	LEVEL_DATA("%s retrieved \"%s\", %ld bytes in %d.%06d seconds\n",
-			   f->client_addr_str, OWQ_pn(&owq).path, size_write, transfer_time.tv_sec, transfer_time.tv_usec);
+	LEVEL_DATA("%s retrieved \"%s\", %ld bytes in "TVformat,
+			   f->client_addr_str, OWQ_pn(&owq).path, size_write, TVvar(&transfer_time) );
 
   exit_retr:
-	if (OWQ_buffer(&owq))
-		free(OWQ_buffer(&owq));
-	if (buf2)
+	OWQ_destroy(&owq); // safe at any speed
+	if (buf2) {
 		free(buf2);
-	if (need_owq_destroy)
-		FS_OWQ_destroy(&owq);
+	}
 	f->file_offset = 0;
 	if (socket_fd != -1) {
 		close(socket_fd);
@@ -1057,7 +1050,8 @@ static void do_stor(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 	struct timeval limit_time = { Globals.timeout_ftp, 0 };
 
 	size_t size_read;
-	int need_pn_destroy = 1;
+	off_t offset = 0 ;
+	char * data_in ;
 	struct parsedname *pn;
 	OWQ_allocate_struct_and_pointer(owq);
 
@@ -1072,16 +1066,16 @@ static void do_stor(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 	socket_fd = -1;
 
 	/* create an absolute name for our file */
-	if (FS_OWQ_create_plus(f->dir, cmd->arg[0].string, NULL, 0, 0, owq)) {
+	if ( BAD( OWQ_create_plus(f->dir, cmd->arg[0].string, owq) ) ) {
 		reply(f, 550, "File does not exist.");
-		need_pn_destroy = 0;
 		goto exit_stor;
 	}
 
 	/* if the last command was a REST command, restart at the */
 	/* requested position in the file                         */
-	if ((f->file_offset_command_number == (f->command_number - 1)))
-		OWQ_offset(owq) = f->file_offset;
+	if ((f->file_offset_command_number == (f->command_number - 1))) {
+		offset = f->file_offset;
+	}
 
 	if (IsDir(pn)) {
 		reply(f, 550, "Error, file is a directory.");
@@ -1098,13 +1092,6 @@ static void do_stor(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 		reply(f, 550, "Error, binary file (type ascii).");
 		goto exit_stor;
 	}
-	size_read = FullFileLength(PN(owq)) - OWQ_offset(owq) + 100;
-	OWQ_buffer(owq) = malloc(size_read);
-
-	if (OWQ_buffer(owq) == NULL) {
-		reply(f, 550, "Out of memory.");
-		goto exit_stor;
-	}
 
 	/* ready to transfer */
 	reply(f, 150, "About to open data connection.");
@@ -1114,17 +1101,29 @@ static void do_stor(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 
 	/* open data path */
 	socket_fd = open_connection(f);
-	if (socket_fd == -1)
+	if (socket_fd == -1) {
 		goto exit_stor;
+	}
 
-	{
+	size_read = FullFileLength(PN(owq)) - OWQ_offset(owq) + 100;
+	data_in = owmalloc(size_read);
+	if (data_in == NULL) {
+		reply(f, 550, "Out of memory.");
+		goto exit_stor;
+	} else {
 		/* we're golden, read the file */
-		ssize_t size_actual = tcp_read(socket_fd, OWQ_buffer(owq), size_read, &limit_time);
-		if (size_actual < 0) {
+		size_t size_actual ;
+		int read_return = tcp_read(socket_fd, (BYTE *) data_in, size_read, &limit_time, &size_actual) ;
+		if (read_return != 0 && read_return != -EAGAIN) {
 			reply(f, 550, "Error reading from data connection; %s.", strerror(errno));
 			goto exit_stor;
 		}
-		OWQ_size(owq) = size_actual;
+		if ( BAD( OWQ_allocate_write_buffer( data_in, size_actual, offset, owq )) ) {
+			owfree(data_in) ;
+			reply(f, 550, "Out of memory.");
+			goto exit_stor;
+		}
+		owfree(data_in) ;
 	}
 
 	watchdog_defer_watched(f->watched);
@@ -1140,35 +1139,23 @@ static void do_stor(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 	gettimeofday(&end_timestamp, NULL);
 
 	/* calculate transfer rate */
-	transfer_time.tv_sec = end_timestamp.tv_sec - start_timestamp.tv_sec;
-	transfer_time.tv_usec = end_timestamp.tv_usec - start_timestamp.tv_usec;
-	while (transfer_time.tv_usec >= 1000000) {
-		transfer_time.tv_sec++;
-		transfer_time.tv_usec -= 1000000;
-	}
-	while (transfer_time.tv_usec < 0) {
-		transfer_time.tv_sec--;
-		transfer_time.tv_usec += 1000000;
-	}
+	timersub( &end_timestamp, &start_timestamp, &transfer_time ) ;
 
 	{
 		/* write to one-wire */
-		int tcp_error = FS_write_postparse(owq);
-		if (tcp_error < 0) {
-			reply(f, 550, "Error writing to file; %s.", strerror(-tcp_error));
+		int one_wire_error = FS_write_postparse(owq);
+		if (one_wire_error < 0) {
+			reply(f, 550, "Error writing to file; %s.", strerror(-one_wire_error));
 			goto exit_stor;
 		}
 	}
 
 	/* note the transfer */
-	LEVEL_DATA("%s stored \"%s\", %ld bytes in %d.%06d seconds\n",
-			   f->client_addr_str, pn->path, OWQ_size(owq), transfer_time.tv_sec, transfer_time.tv_usec);
+	LEVEL_DATA("%s stored \"%s\", %ld bytes in "TVformat,
+			   f->client_addr_str, pn->path, OWQ_size(owq), TVvar(&transfer_time) );
 
   exit_stor:
-	if (OWQ_buffer(owq))
-		free(OWQ_buffer(owq));
-	if (need_pn_destroy)
-		FS_OWQ_destroy(owq);
+	OWQ_destroy(owq);
 	f->file_offset = 0;
 	if (socket_fd != -1) {
 		close(socket_fd);
@@ -1193,6 +1180,7 @@ static int open_connection(struct ftp_session_s *f)
 			reply(f, 425, "Error creating socket; %s.", strerror(errno));
 			return -1;
 		}
+		fcntl (socket_fd, F_SETFD, FD_CLOEXEC); // for safe forking
 		if (connect(socket_fd, (struct sockaddr *) &f->data_port, sizeof(sockaddr_storage_t)) != 0) {
 			reply(f, 425, "Error connecting; %s.", strerror(errno));
 			close(socket_fd);
@@ -1398,12 +1386,12 @@ static void do_size(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 	} else {
 
 		/* get the file information */
-		if (FS_ParsedNamePlus(f->dir, cmd->arg[0].string, &pn)) {
+		if ( FS_ParsedNamePlus(f->dir, cmd->arg[0].string, &pn) != 0 ) {
 			reply(f, 550, "Bad file specification");
 		} else {
 
 			/* verify that the file is not a directory */
-			if (pn.selected_device == NULL || pn.selected_filetype == NULL) {
+			if (pn.selected_device == NO_DEVICE || pn.selected_filetype == NO_FILETYPE) {
 				reply(f, 550, "File is a directory, SIZE command not valid.");
 			} else {
 				filesize = FullFileLength(&pn);
@@ -1426,9 +1414,9 @@ struct tm *gmtime_r(const time_t * timep, struct tm *timeptr)
 {
 	static pthread_mutex_t time_lock = PTHREAD_MUTEX_INITIALIZER;
 
-	pthread_mutex_lock(&time_lock);
+	_MUTEX_LOCK(time_lock);
 	*timeptr = *(gmtime(timep));
-	pthread_mutex_unlock(&time_lock);
+	_MUTEX_UNLOCK(time_lock);
 	return timeptr;
 }
 #endif							/* HAVE_GMTIME_R */
@@ -1450,7 +1438,7 @@ static void do_mdtm(struct ftp_session_s *f, const struct ftp_command_s *cmd)
 	get_absolute_fname(full_path, sizeof(full_path), f->dir, file_name);
 
 	/* get the file information */
-	if (FS_fstat(full_path, &stat_buf)) {
+	if (FS_fstat(full_path, &stat_buf) < 0) {
 		reply(f, 550, "Error getting file status; %s.", strerror(errno));
 	} else {
 		gmtime_r(&stat_buf.st_mtime, &mtime);
@@ -1487,7 +1475,7 @@ static void send_readme(const struct ftp_session_s *f, int code)
 static void netscape_hack(int file_descriptor)
 {
 	fd_set readfds;
-	struct timeval ns_timeout;
+	struct timeval ns_timeout = {15, 0 } ; // 15 seconds
 	int select_ret;
 	char c;
 
@@ -1496,11 +1484,9 @@ static void netscape_hack(int file_descriptor)
 	shutdown(file_descriptor, 1);
 	FD_ZERO(&readfds);
 	FD_SET(file_descriptor, &readfds);
-	ns_timeout.tv_sec = 15;
-	ns_timeout.tv_usec = 0;
 	select_ret = select(file_descriptor + 1, &readfds, NULL, NULL, &ns_timeout);
 	if (select_ret > 0) {
-		read(file_descriptor, &c, 1);
+		ignore_result = read(file_descriptor, &c, 1); // ignore warning
 	}
 }
 
