@@ -1,5 +1,5 @@
 /*
-$Id: owhttpd.h,v 1.17 2008/01/30 13:15:11 d1mag Exp $
+$Id: owhttpd.h,v 1.22 2011/08/07 22:56:07 alfille Exp $
    OWFS and OWHTTPD
    one-wire file system and
    one-wire web server
@@ -45,16 +45,21 @@ void HTTPheader(FILE * out, const char *head);
 void HTTPfoot(FILE * out);
 
 /* in owhttpd_write.c */
+void PostData(struct one_wire_query *owq);
 void ChangeData(struct one_wire_query *owq);
 
 /* in owhttpd_read.c */
-void ShowDevice(FILE * out, const struct parsedname *const pn);
+void ShowDevice(FILE * out, struct parsedname *const pn);
 
 /* in owhttpd_dir.c */
-void ShowDir(FILE * out, const struct parsedname *const pn);
+void ShowDir(FILE * out, struct parsedname * pn);
 int Backup(const char *path);
 
 /* in ow_favicon.c */
 void Favicon(FILE * out);
 
+/* in owhttpd_escape */
+void httpunescape(BYTE * httpstr) ;
+char * httpescape( char * original_string ) ;
+
 #endif							/* OWHTTPD_H */
