@@ -47,8 +47,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <config.h>
 #include "owfs_config.h"
 
-#include "ow_debug.h"
-
 #ifdef HAVE_PTHREAD
 #include <pthread.h>
 #endif
@@ -64,9 +62,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <errno.h>
 #include <netdb.h>
 #include "compat_netdb.h"
-#ifdef HAVE_RESOLV_H
-#include <resolv.h>
-#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -79,6 +75,13 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <sys/utsname.h>
 #include <net/if.h>
 
+#ifdef HAVE_RESOLV_H
+#include <resolv.h>
+#endif
+
+#include "ow_debug.h"
+#include "ow_alloc.h"
+
 /* The following declarations and definitions have been removed from
  *    the public header since we don't want people to use them.  */
 #define AI_V4MAPPED     0x0008	/* IPv4-mapped addresses are acceptable.  */
@@ -264,7 +267,7 @@ struct hostent *gethostbyname_r(const char *name, struct hostent *result, char *
 	(void) buflen;				// not used
 
 #ifdef HAVE_PTHREAD
-	my_pthread_mutex_lock(&gethostbyname_lock);
+	_MUTEX_LOCK(gethostbyname_lock);
 #endif
 	res = gethostbyname(name);
 	if (res) {
@@ -273,7 +276,7 @@ struct hostent *gethostbyname_r(const char *name, struct hostent *result, char *
 		*h_errnop = errno;
 	}
 #ifdef HAVE_PTHREAD
-	my_pthread_mutex_unlock(&gethostbyname_lock);
+	_MUTEX_UNLOCK(gethostbyname_lock);
 #endif
 	return res;
 }
@@ -290,13 +293,13 @@ struct servent *getservbyname_r(const char *name, const char *proto, struct serv
 	(void) buflen;				// not used
 
 #ifdef HAVE_PTHREAD
-	my_pthread_mutex_lock(&getservbyname_lock);
+	_MUTEX_LOCK(getservbyname_lock);
 #endif
 	res = getservbyname(name, proto);
 	if (res)
 		memcpy(result, res, sizeof(struct servent));
 #ifdef HAVE_PTHREAD
-	my_pthread_mutex_unlock(&getservbyname_lock);
+	_MUTEX_UNLOCK(getservbyname_lock);
 #endif
 	return res;
 }
@@ -468,7 +471,7 @@ struct hostent *gethostbyaddr_r(const char *name, int len, int type, struct host
 	(void) buflen;				// not used
 
 #ifdef HAVE_PTHREAD
-	my_pthread_mutex_lock(&gethostbyaddr_lock);
+	_MUTEX_LOCK(gethostbyaddr_lock);
 #endif
 	res = gethostbyaddr(name, len, type);
 	if (res) {
@@ -477,7 +480,7 @@ struct hostent *gethostbyaddr_r(const char *name, int len, int type, struct host
 		*h_errnop = errno;
 	}
 #ifdef HAVE_PTHREAD
-	my_pthread_mutex_unlock(&gethostbyaddr_lock);
+	_MUTEX_UNLOCK(gethostbyaddr_lock);
 #endif
 	return res;
 }
