@@ -1,5 +1,5 @@
 /*
-$Id: ow_ds9097U.c,v 1.111 2009/04/05 12:01:12 d1mag Exp $
+$Id: ow_ds9097U.c,v 1.178 2012/04/07 18:41:41 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
@@ -9,31 +9,56 @@ $Id: ow_ds9097U.c,v 1.111 2009/04/05 12:01:12 d1mag Exp $
     1wire/iButton system from Dallas Semiconductor
 */
 
+/* This is the busmaster code for the DS2480B-based DS9097U
+ * The serial adapter from Dallas Maxim
+ */ 
+ 
+/* Multimaster:
+ * In general the link is DS9097U a multimaster design (only one bus master per serial or telnet port)
+ * None the less, all settings are assigned to the head line
+ */
+ 
+ 
+
+
 #include <config.h>
 #include "owfs_config.h"
 #include "ow.h"
 #include "ow_counters.h"
 #include "ow_connection.h"
 
-static int DS2480_next_both(struct device_search *ds, const struct parsedname *pn);
-//static int DS2480_databit(int sendbit, int *getbit, const struct parsedname *pn);
-static int DS2480_reset(const struct parsedname *pn);
-static int DS2480_big_reset(const struct parsedname *pn) ;
-static int DS2480_read(BYTE * buf, const size_t size, const struct parsedname *pn);
-static int DS2480_write(const BYTE * buf, const size_t size, const struct parsedname *pn);
-static int DS2480_PowerByte(const BYTE byte, BYTE * resp, const UINT delay, const struct parsedname *pn);
-static int DS2480_ProgramPulse(const struct parsedname *pn);
-static int DS2480_sendout_cmd(const BYTE * cmd, const size_t len, const struct parsedname *pn);
-static int DS2480_sendback_cmd(const BYTE * cmd, BYTE * resp, const size_t len, const struct parsedname *pn);
-static int DS2480_sendback_data(const BYTE * data, BYTE * resp, const size_t len, const struct parsedname *pn);
+static RESET_TYPE DS2480_reset(const struct parsedname *pn);
+static enum search_status DS2480_next_both(struct device_search *ds, const struct parsedname *pn);
+static GOOD_OR_BAD DS2480_PowerByte(const BYTE byte, BYTE * resp, const UINT delay, const struct parsedname *pn);
+static GOOD_OR_BAD DS2480_PowerBit(const BYTE byte, BYTE * resp, const UINT delay, const struct parsedname *pn);
+static GOOD_OR_BAD DS2480_ProgramPulse(const struct parsedname *pn);
+static GOOD_OR_BAD DS2480_sendback_data(const BYTE * data, BYTE * resp, const size_t len, const struct parsedname *pn);
+static GOOD_OR_BAD DS2480_sendback_bits(const BYTE * databits, BYTE * respbits, const size_t len, const struct parsedname * pn);
+static GOOD_OR_BAD DS2480_reconnect(const struct parsedname * pn);
+static void DS2480_close(struct connection_in *in) ;
+
 static void DS2480_setroutines(struct connection_in *in);
-static int DS2480_configuration_write(BYTE parameter_code, BYTE value_code, const struct parsedname *pn);
-static int DS2480_configuration_read(BYTE parameter_code, BYTE value_code, const struct parsedname *pn);
-static int DS2480_stop_pulse(BYTE * response, const struct parsedname *pn);
-static int DS2480_reset_once(const struct parsedname *pn) ;
-static int DS2480_set_baud(const struct parsedname *pn) ;
-static void DS2480_set_baud_control(const struct parsedname *pn) ;
-static BYTE DS2480b_speed_byte( const struct parsedname * pn ) ;
+
+static GOOD_OR_BAD DS2480_detect_serial(struct connection_in *in) ;
+static RESET_TYPE DS2480_reset_in(struct connection_in * in);
+static GOOD_OR_BAD DS2480_initialize_repeatedly(struct connection_in * in);
+static GOOD_OR_BAD DS2480_big_reset(struct connection_in * in) ;
+static GOOD_OR_BAD DS2480_big_reset_serial(struct connection_in * in) ;
+static void DS2480_adapter(struct connection_in *in) ;
+static GOOD_OR_BAD DS2480_big_configuration(struct connection_in * in) ;
+static GOOD_OR_BAD DS2480_read(BYTE * buf, const size_t size, struct connection_in * in);
+static GOOD_OR_BAD DS2480_write(const BYTE * buf, const size_t size, struct connection_in * in);
+static GOOD_OR_BAD DS2480_sendout_cmd(const BYTE * cmd, const size_t len, struct connection_in * in);
+static GOOD_OR_BAD DS2480_sendback_cmd(const BYTE * cmd, BYTE * resp, const size_t len, struct connection_in * in);
+static GOOD_OR_BAD DS2480_configuration_write(BYTE parameter_code, BYTE value_code, struct connection_in * in);
+static GOOD_OR_BAD DS2480_configuration_read(BYTE parameter_code, BYTE value_code, struct connection_in * in);
+static GOOD_OR_BAD DS2480_stop_pulse(BYTE * response, struct connection_in * in);
+static RESET_TYPE DS2480_reset_once(struct connection_in * in) ;
+static GOOD_OR_BAD DS2480_set_baud(struct connection_in * in) ;
+static void DS2480_set_baud_control(struct connection_in * in) ;
+static BYTE DS2480b_speed_byte( struct connection_in * in ) ;
+static void DS2480_flush( const struct connection_in * in ) ;
+static void DS2480_slurp( struct connection_in * in ) ;
 
 static void DS2480_setroutines(struct connection_in *in)
 {
@@ -41,17 +66,22 @@ static void DS2480_setroutines(struct connection_in *in)
 	in->iroutines.reset = DS2480_reset;
 	in->iroutines.next_both = DS2480_next_both;
 	in->iroutines.PowerByte = DS2480_PowerByte;
+	in->iroutines.PowerBit = DS2480_PowerBit;
 	in->iroutines.ProgramPulse = DS2480_ProgramPulse;
+//	in->iroutines.sendback_data = NO_SENDBACKDATA_ROUTINE;
 	in->iroutines.sendback_data = DS2480_sendback_data;
-//    in->iroutines.sendback_bits = ;
-	in->iroutines.select = NULL;
-	in->iroutines.reconnect = DS2480_big_reset ;
-	in->iroutines.close = COM_close;
-	in->iroutines.transaction = NULL;
-	in->iroutines.flags = 0;
+    in->iroutines.sendback_bits = DS2480_sendback_bits;
+	in->iroutines.select = NO_SELECT_ROUTINE;
+	in->iroutines.select_and_sendback = NO_SELECTANDSENDBACK_ROUTINE;
+	in->iroutines.reconnect = DS2480_reconnect ;
+	in->iroutines.close = DS2480_close;
+	in->iroutines.flags = ADAP_FLAG_default;
 	in->bundling_length = UART_FIFO_SIZE;
 }
 
+// Number of times to try init (from digitemp code))
+#define DS9097U_INIT_CYCLES   3
+
 /* --------------------------- */
 /* DS2480 defines from PDkit   */
 /* --------------------------- */
@@ -84,7 +114,7 @@ static void DS2480_setroutines(struct connection_in *in)
 
 // Command or config bit
 #define CMD_COMM                       0x81
-#define CMD_COMM_RESPintONSE              0x80
+#define CMD_COMM_RESPONSE              0x80
 #define CMD_CONFIG                     0x01
 #define CMD_CONFIG_RESPONSE            0x00
 
@@ -201,38 +231,60 @@ static void DS2480_setroutines(struct connection_in *in)
 #define MAX_SEND_SIZE			64
 
 /* Reset and detect a DS2480B */
-/* returns 0=good
-   DS2480 sendback error
-   COM_write error
-   -EINVAL baudrate error
-   If no detection, try a DS9097 passive port */
+/* returns 0=good */
 // bus locking at a higher level
-int DS2480_detect(struct connection_in *in)
+GOOD_OR_BAD DS2480_detect(struct connection_in *in)
 {
-	struct parsedname pn;
-	int ret;
-
-	FS_ParsedName(NULL, &pn);	// minimal parsename -- no destroy needed
-	pn.selected_connection = in;
-
 	/* Set up low-level routines */
 	DS2480_setroutines(in);
 
-	in->speed = bus_speed_slow ;
-	in->flex = Globals.serial_flextime ? bus_yes_flex : bus_no_flex ;
+	in->overdrive = 0 ;
+	in->flex = Globals.serial_flextime ;
 
 	// Now set desired baud and polarity
 	// BUS_reset will do the actual changes
-	in->connin.serial.reverse_polarity = Globals.serial_reverse ;
-	in->baud = Globals.baud ;
+	in->master.serial.reverse_polarity = Globals.serial_reverse ;
+	COM_set_standard( in ) ; // standard COM port settings
+	
+	// first pass with hardware flow control
+	RETURN_GOOD_IF_GOOD( DS2480_detect_serial(in) ) ;
 
-	ret = DS2480_big_reset(&pn) ;
-	if ( ret ) {
-		return ret ;
+	SOC(in)->flow = flow_second; // flow control
+	RETURN_BAD_IF_BAD(COM_change(in)) ;
+	return DS2480_detect_serial(in) ;
+}
+
+// setting for serial port already made
+static GOOD_OR_BAD DS2480_detect_serial(struct connection_in *in)
+{
+	SOC(in)->state = cs_virgin ;
+	if ( BAD(DS2480_initialize_repeatedly(in)) ) {
+		LEVEL_DEBUG("Could not initilize the DS9097U even after several tries") ;
+		COM_close(in) ;
+		return gbBAD ;
 	}
 
 	in->busmode = bus_serial;
 
+	DS2480_adapter(in) ;
+	return gbGOOD ;
+}
+
+// Make several attempts to initialize -- based on Digitemp example
+static GOOD_OR_BAD DS2480_initialize_repeatedly(struct connection_in * in)
+{
+	int init_cycles ;
+
+	for ( init_cycles = 0 ; init_cycles < DS9097U_INIT_CYCLES ; ++init_cycles ) {
+		LEVEL_DEBUG("Attempt %d of %d to initialize the DS9097U",init_cycles,DS9097U_INIT_CYCLES) ;
+		RETURN_GOOD_IF_GOOD( DS2480_big_reset(in) ) ;
+	}
+
+	return gbBAD ;
+}
+
+static void DS2480_adapter(struct connection_in *in)
+{
 	// in->Adapter is set in DS2480_reset from some status bits
 	switch (in->Adapter) {
 	case adapter_DS9097U2:
@@ -243,162 +295,190 @@ int DS2480_detect(struct connection_in *in)
 	case adapter_LINK_10:
 	case adapter_LINK_11:
 	case adapter_LINK_12:
+	case adapter_LINK_13:
+	case adapter_LINK_14:
+	case adapter_LINK_other:
 		in->adapter_name = "LINK(emulate mode)";
 		break;
 	default:
-		return -ENODEV;
+		in->adapter_name = "DS2480B based";
+		break;
 	}
-	return 0;
+}
+
+static GOOD_OR_BAD DS2480_reconnect(const struct parsedname * pn)
+{
+	LEVEL_DEBUG("Attempting reconnect on %s",SAFESTRING(SOC(pn->selected_connection)->devicename));
+	return DS2480_big_reset(pn->selected_connection) ;
 }
 
 // do the com port and configuration stuff
-static int DS2480_big_reset(const struct parsedname *pn)
+static GOOD_OR_BAD DS2480_big_reset(struct connection_in * in)
+{
+	switch (SOC(in)->type) {
+		case ct_telnet:
+			SOC(in)->timeout.tv_sec = Globals.timeout_network ;
+			SOC(in)->timeout.tv_usec = 0 ;
+			return DS2480_big_reset_serial(in) ;
+
+		case ct_serial:
+		default:
+			SOC(in)->timeout.tv_sec = Globals.timeout_serial ;
+			SOC(in)->timeout.tv_usec = 0 ;
+
+			SOC(in)->flow = flow_none ;
+			RETURN_GOOD_IF_GOOD( DS2480_big_reset_serial(in)) ;
+
+			SOC(in)->flow = flow_none ;
+			RETURN_GOOD_IF_GOOD( DS2480_big_reset_serial(in)) ;
+
+			SOC(in)->flow = flow_hard ;
+			RETURN_GOOD_IF_GOOD( DS2480_big_reset_serial(in)) ;
+
+			return gbBAD ;
+	}
+}
+// do the com port and configuration stuff
+static GOOD_OR_BAD DS2480_big_reset_serial(struct connection_in * in)
 {
-	int ret ;
-	BYTE single_bit = CMD_COMM | BITPOL_ONE |  DS2480b_speed_byte(pn) ;
-	BYTE single_bit_response ;
 	BYTE reset_byte = (BYTE) ( CMD_COMM | FUNCTSEL_RESET | SPEEDSEL_STD );
 
 	// Open the com port in 9600 Baud.
-	if (COM_open(pn->selected_connection)) {
-		return -ENODEV;
-	}
+	RETURN_BAD_IF_BAD(COM_open(in)) ;
 
 	// send a break to reset the DS2480
-	COM_break(pn->selected_connection);
+	COM_break(in);
 
 	// It's in command mode now
-	pn->selected_connection->connin.serial.mode = ds2480b_command_mode ;
+	in->master.serial.mode = ds2480b_command_mode ;
 
 	// send the timing byte (A reset command at 9600 baud)
-	DS2480_write( &reset_byte, 1, pn ) ;
+	DS2480_write( &reset_byte, 1, in ) ;
 
 	// delay to let line settle
 	UT_delay(4);
 	// flush the buffers
-	COM_flush(pn->selected_connection);
+	DS2480_flush(in);
 	// ignore response
-	COM_slurp( pn->selected_connection->file_descriptor ) ;
+	DS2480_slurp( in ) ;
 	// Now set desired baud and polarity
 	// BUS_reset will do the actual changes
-	pn->selected_connection->changed_bus_settings = 1 ; // Force a mode change
+	in->changed_bus_settings = 1 ; // Force a mode change
 	// Send a reset again
-	BUS_reset(pn) ;
+	LEVEL_DEBUG("Send the initial reset to the bus master.");
+	DS2480_reset_in(in) ;
 
 	// delay to let line settle
-	UT_delay(4);
+	UT_delay(400);
 	// flush the buffers
-	COM_flush(pn->selected_connection);
+	DS2480_flush(in);
 	// ignore response
-	COM_slurp( pn->selected_connection->file_descriptor ) ;
+	DS2480_slurp( in ) ;
+
+	// Now set desired baud and polarity
+	return DS2480_big_configuration(in) ;
+}
+
+// do the configuration stuff
+static GOOD_OR_BAD DS2480_big_configuration(struct connection_in * in)
+{
+	BYTE single_bit = CMD_COMM | BITPOL_ONE |  DS2480b_speed_byte(in) ;
+	BYTE single_bit_response ;
+
 	// Now set desired baud and polarity
 	// BUS_reset will do the actual changes
-	pn->selected_connection->changed_bus_settings = 1 ; // Force a mode change
+	in->changed_bus_settings = 1 ; // Force a mode change
 	// Send a reset again
-	BUS_reset(pn) ;
+	DS2480_reset_in(in) ;
 
 	// delay to let line settle
 	UT_delay(4);
 
 	// default W1LT = 10us (write-1 low time)
-	if (DS2480_configuration_write(PARMSEL_WRITE1LOW, PARMSET_Write10us, pn)) {
-		return -EINVAL;
-	}
+	RETURN_BAD_IF_BAD(DS2480_configuration_write(PARMSEL_WRITE1LOW, PARMSET_Write10us, in)) ;
+
 	// default DSO/WORT = 8us (data sample offset / write 0 recovery time )
-	if (DS2480_configuration_write(PARMSEL_SAMPLEOFFSET, PARMSET_SampOff8us, pn)) {
-		return -EINVAL;
-	}
+	RETURN_BAD_IF_BAD(DS2480_configuration_write(PARMSEL_SAMPLEOFFSET, PARMSET_SampOff8us, in)) ;
+
 	// Strong pullup duration = infinite
-	if (DS2480_configuration_write(PARMSEL_5VPULSE, PARMSET_infinite, pn)) {
-		return -EINVAL;
-	}
+	RETURN_BAD_IF_BAD(DS2480_configuration_write(PARMSEL_5VPULSE, PARMSET_infinite, in)) ;
+
 	// Program pulse duration = 512usec
-	if (DS2480_configuration_write(PARMSEL_12VPULSE, PARMSET_512us, pn)) {
-		return -EINVAL;
-	}
+	RETURN_BAD_IF_BAD(DS2480_configuration_write(PARMSEL_12VPULSE, PARMSET_512us, in)) ;
 
 	// Send a single bit
 	// The datasheet wants this
-	if (DS2480_sendback_cmd(&single_bit, &single_bit_response, 1, pn)) {
-		return -EIO;
-	}
+	RETURN_BAD_IF_BAD(DS2480_sendback_cmd(&single_bit, &single_bit_response, 1, in)) ;
 
 	/* Apparently need to reset again to get the version number properly */
-	if ((ret = DS2480_reset(pn))<0) {
-		return ret;
-	}
-
-	return 0 ;
+	return gbRESET( DS2480_reset_in(in)  ) ;
 }
 
 // configuration of DS2480B -- parameter code is already shifted in the defines (by 4 bites)
 // configuration of DS2480B -- value code is already shifted in the defines (by 1 bit)
-static int DS2480_configuration_write(BYTE parameter_code, BYTE value_code, const struct parsedname *pn)
+static GOOD_OR_BAD DS2480_configuration_write(BYTE parameter_code, BYTE value_code, struct connection_in * in)
 {
 	BYTE send_code = CMD_CONFIG | parameter_code | value_code;
 	BYTE expected_response = CMD_CONFIG_RESPONSE | parameter_code | value_code;
 	BYTE actual_response;
 
-	if (DS2480_sendback_cmd(&send_code, &actual_response, 1, pn)) {
-		return -EIO;
-	}
+	RETURN_BAD_IF_BAD(DS2480_sendback_cmd(&send_code, &actual_response, 1, in)) ;
+
 	if (expected_response == actual_response) {
-		return 0;
+		return gbGOOD;
 	}
-    LEVEL_DEBUG("wrong response (%.2X not %.2X)\n",actual_response,expected_response) ;
-    return -EINVAL;
+	LEVEL_DEBUG("wrong response (%.2X not %.2X)",actual_response,expected_response) ;
+	return gbBAD;
 }
 
 // configuration of DS2480B -- parameter code is already shifted in the defines (by 4 bites)
 // configuration of DS2480B -- value code is already shifted in the defines (by 1 bit)
-static int DS2480_configuration_read(BYTE parameter_code, BYTE value_code, const struct parsedname *pn)
+static GOOD_OR_BAD DS2480_configuration_read(BYTE parameter_code, BYTE value_code, struct connection_in * in)
 {
 	BYTE send_code = CMD_CONFIG | PARMSEL_PARMREAD | (parameter_code>>3);
 	BYTE expected_response = CMD_CONFIG_RESPONSE | PARMSEL_PARMREAD | value_code;
 	BYTE actual_response;
 
-	if (DS2480_sendback_cmd(&send_code, &actual_response, 1, pn)) {
-		return -EIO;
-	}
+	RETURN_BAD_IF_BAD( DS2480_sendback_cmd(&send_code, &actual_response, 1, in)) ;
 	if (expected_response == actual_response) {
-		return 0;
+		return gbGOOD;
 	}
-    LEVEL_DEBUG("wrong response (%.2X not %.2X)\n",actual_response,expected_response) ;
-    return -EINVAL;
+	LEVEL_DEBUG("wrong response (%.2X not %.2X)",actual_response,expected_response) ;
+	return gbBAD;
 }
 
 // configuration of DS2480B -- parameter code is already shifted in the defines (by 4 bites)
 // configuration of DS2480B -- value code is already shifted in the defines (by 1 bit)
-// Set Baud rate -- can't use DS2480_conficuration_code because return byte is in a different speed
-static void DS2480_set_baud_control(const struct parsedname *pn)
+// Set Baud rate -- can't use DS2480_configuration_code because return byte is at a different speed
+static void DS2480_set_baud_control(struct connection_in * in)
 {
 	// restrict allowable baud rates based on device capabilities
-	OW_BaudRestrict( &(pn->selected_connection->baud), B9600, B19200, B57600, B115200, 0 ) ;
+	COM_BaudRestrict( &(SOC(in)->baud), B9600, B19200, B57600, B115200, 0 ) ;
 
-	if ( DS2480_set_baud(pn) == 0 ) {
+	if ( GOOD( DS2480_set_baud(in) ) ) {
 		return ;
 	}
-	LEVEL_DEBUG("Failed first attempt at resetting baud rate of bus master %s\n",SAFESTRING(pn->selected_connection->name)) ;
+	LEVEL_DEBUG("Failed first attempt at resetting baud rate of bus master %s",SAFESTRING(SOC(in)->devicename)) ;
 
-	if ( DS2480_set_baud(pn) == 0 ) {
+	if ( GOOD( DS2480_set_baud(in) ) ) {
 		return ;
 	}
-	LEVEL_DEBUG("Failed second attempt at resetting baud rate of bus master %s\n",SAFESTRING(pn->selected_connection->name)) ;
+	LEVEL_DEBUG("Failed second attempt at resetting baud rate of bus master %s",SAFESTRING(SOC(in)->devicename)) ;
 
 	// uh oh -- undefined state -- not sure what the bus speed is.
-	pn->selected_connection->reconnect_state = reconnect_error ;
-	pn->selected_connection->baud = B9600 ;
-	++pn->selected_connection->changed_bus_settings ;
+	in->reconnect_state = reconnect_error ;
+	SOC(in)->baud = B9600 ;
+	++in->changed_bus_settings ;
 	return;
 }
 
-static int DS2480_set_baud(const struct parsedname *pn)
+static GOOD_OR_BAD DS2480_set_baud(struct connection_in * in)
 {
-	BYTE value_code ;
+	BYTE value_code = 0 ;
 	BYTE send_code ;
 
 	// Find rate parameter
-	switch ( pn->selected_connection->baud ) {
+	switch ( SOC(in)->baud ) {
 		case B9600:
 			value_code = PARMSET_9600 ;
 			break ;
@@ -417,42 +497,42 @@ static int DS2480_set_baud(const struct parsedname *pn)
 			value_code = PARMSET_115200 ;
 			break ;
 #endif
+		default:
+			SOC(in)->baud = B9600 ;
+			value_code = PARMSET_9600 ;
+			break ;
 	}
 
 	// Add polarity
-	if ( pn->selected_connection->connin.serial.reverse_polarity ) {
+	if ( in->master.serial.reverse_polarity ) {
 		value_code |= PARMSET_REVERSE_POLARITY ;
 	}
 	send_code = CMD_CONFIG | PARMSEL_BAUDRATE | value_code;
 
 	// Send configuration change
-	COM_flush(pn->selected_connection);
+	DS2480_flush(in);
 	UT_delay(5);
-	if (DS2480_sendout_cmd(&send_code, 1, pn)) {
-		// uh oh -- undefined state -- not sure what the bus speed is.
-		return 1;
-	}
+
+	// uh oh -- undefined state -- not sure what the bus speed is.
+	RETURN_BAD_IF_BAD(DS2480_sendout_cmd(&send_code, 1, in)) ;
 
 	// Change OS view of rate
 	UT_delay(5);
-	COM_speed(pn->selected_connection->baud,pn->selected_connection) ;
+	COM_change(in) ;
 	UT_delay(5);
-	COM_slurp(pn->selected_connection->file_descriptor);
+	DS2480_slurp(in);
 
 	// Check rate
-	if ( DS2480_configuration_read( PARMSEL_BAUDRATE, value_code, pn ) ) {
-		// Can't read new setting
-		return 1 ;
-	}
+	RETURN_BAD_IF_BAD( DS2480_configuration_read( PARMSEL_BAUDRATE, value_code, in ) ) ;
 
-	return 0 ;
+	return gbGOOD ;
 }
 
-static BYTE DS2480b_speed_byte( const struct parsedname * pn )
+static BYTE DS2480b_speed_byte( struct connection_in * in )
 {
-	if ( pn->selected_connection->speed == bus_speed_overdrive ) {
+	if ( in->overdrive ) {
 		return SPEEDSEL_OD ;
-	} else if ( pn->selected_connection->flex == bus_yes_flex ) {
+	} else if ( in->flex ) {
 		return SPEEDSEL_FLEX ;
 	} else {
 		return SPEEDSEL_STD ;
@@ -465,97 +545,98 @@ static BYTE DS2480b_speed_byte( const struct parsedname * pn )
 //          This routine will not function correctly on some
 //          Alarm reset types of the DS1994/DS1427/DS2404 with
 //          Rev 1,2, and 3 of the DS2480/DS2480B.
-/* return 0=good
-          1=short
-          <0 error
- */
-static int DS2480_reset(const struct parsedname *pn)
-{
-	int ret ;
-
-	if (pn->selected_connection->changed_bus_settings > 0) {
-		--pn->selected_connection->changed_bus_settings ;
-		DS2480_set_baud_control(pn);	// reset paramters
-	}
-
-	ret = DS2480_reset_once(pn) ;
-
-	// Some kind of problem (not including bus short)
-	// Try simple fixes
-	if ( ret < 0 ) {
-		BYTE dummy[1] ; //we don't check the pulse stop, since it's only a whim.
-		// perhaps the DS9097U wasn't in command mode, so missed the reset
-		// force a mode switch
-		pn->selected_connection->connin.serial.mode = ds2480b_data_mode ;
-		// Also make sure no power or programming pulse active
-		DS2480_stop_pulse(dummy,pn) ;
-		// And now reset again
-		ret = DS2480_reset_once(pn) ;
-	}
-	return ret ;
+static RESET_TYPE DS2480_reset(const struct parsedname *pn) {
+	return DS2480_reset_in( pn->selected_connection ) ;
 }
 
-static int DS2480_reset_once(const struct parsedname *pn)
+static RESET_TYPE DS2480_reset_in(struct connection_in * in)
 {
-	int ret = 0;
-	BYTE reset_byte = (BYTE) ( CMD_COMM | FUNCTSEL_RESET | DS2480b_speed_byte(pn) );
+	if ( in->changed_bus_settings != 0) {
+		in->changed_bus_settings = 0 ;
+		DS2480_set_baud_control(in);	// reset paramters
+	}
+
+	switch( DS2480_reset_once(in) ) {
+		case BUS_RESET_OK:
+			return BUS_RESET_OK ;
+		case BUS_RESET_SHORT:
+			return BUS_RESET_SHORT;
+		case BUS_RESET_ERROR:
+		default: {
+			// Some kind of reset problem (not including bus short)
+			// Try simple fixes
+			BYTE dummy[1] ; //we don't check the pulse stop, since it's only a whim.
+			// perhaps the DS9097U wasn't in command mode, so missed the reset
+			// force a mode switch
+			in->master.serial.mode = ds2480b_data_mode ;
+			// Also make sure no power or programming pulse active
+			DS2480_stop_pulse(dummy,in) ;
+			// And now reset again
+			return DS2480_reset_once(in) ;
+		}
+	}
+}
+
+static RESET_TYPE DS2480_reset_once(struct connection_in * in)
+{
+	BYTE reset_byte = (BYTE) ( CMD_COMM | FUNCTSEL_RESET | DS2480b_speed_byte(in) );
 	BYTE reset_response ;
 
-	//printf("DS2480_reset\n");
 	// flush the buffers
-	COM_flush(pn->selected_connection);
+	DS2480_flush(in);
 
 	// send the packet
 	// read back the 1 byte response
-	if (DS2480_sendback_cmd(&reset_byte, &reset_response, 1, pn)) {
-		return -EIO;
+	if ( BAD(DS2480_sendback_cmd(&reset_byte, &reset_response, 1, in)) ) {
+		return BUS_RESET_ERROR;
 	}
-	/* The adapter type is encode in this response byte */
-	/* The known values coorespond to the types in enum adapter_type */
+
+	/* The adapter type is encoded in this response byte */
+	/* The known values correspond to the types in enum adapter_type */
 	/* Other values are assigned for adapters that don't have this hardcoded value */
-	pn->selected_connection->Adapter = (reset_response & RB_CHIPID_MASK) >> 2;
+	in->Adapter = (reset_response & RB_CHIPID_MASK) >> 2;
 
 	switch (reset_response & RB_RESET_MASK) {
 	case RB_1WIRESHORT:
-		ret = BUS_RESET_SHORT;
-		break;
+		return BUS_RESET_SHORT;
 	case RB_NOPRESENCE:
-		ret = BUS_RESET_OK;
-		pn->selected_connection->AnyDevices = 0;
-		break;
+		in->AnyDevices = anydevices_no ;
+		return BUS_RESET_OK;
 	case RB_PRESENCE:
 	case RB_ALARMPRESENCE:
-		ret = BUS_RESET_OK;
-		pn->selected_connection->AnyDevices = 1;
+		in->AnyDevices = anydevices_yes ;
 		// check if programming voltage available
-		pn->selected_connection->ProgramAvailable = ((reset_response & PARMSEL_12VPULSE) == PARMSEL_12VPULSE);
-		if (pn->selected_connection->ds2404_compliance) {
-			// extra delay for alarming DS1994/DS2404 complience
-			UT_delay(5);
-		}
-		COM_flush(pn->selected_connection);
-		break;
-
+		in->ProgramAvailable = ((reset_response & PARMSEL_12VPULSE) == PARMSEL_12VPULSE);
+		DS2480_flush(in);
+		return BUS_RESET_OK;
+	default:
+		return BUS_RESET_ERROR; // should never happen
 	}
-	return ret;
 }
 
 /* search = normal and alarm */
-static int DS2480_next_both(struct device_search *ds, const struct parsedname *pn)
+static enum search_status DS2480_next_both(struct device_search *ds, const struct parsedname *pn)
 {
-	int ret;
 	int mismatched;
 	BYTE sn[8];
 	BYTE bitpairs[16];
-	BYTE searchon = (BYTE) ( CMD_COMM | FUNCTSEL_SEARCHON | DS2480b_speed_byte(pn) );
-	BYTE searchoff = (BYTE) ( CMD_COMM | FUNCTSEL_SEARCHOFF | DS2480b_speed_byte(pn) );
+	struct connection_in * in = pn->selected_connection ;
+	BYTE searchon = (BYTE) ( CMD_COMM | FUNCTSEL_SEARCHON | DS2480b_speed_byte(in) );
+	BYTE searchoff = (BYTE) ( CMD_COMM | FUNCTSEL_SEARCHOFF | DS2480b_speed_byte(in) );
 	int i;
 
-	if (!(pn->selected_connection->AnyDevices)) {
-		ds->LastDevice = 1;
-	}
 	if (ds->LastDevice) {
-		return -ENODEV;
+		return search_done;
+	}
+
+	if ( BAD( BUS_select(pn) ) ) {
+		return search_error;
+	}
+
+	// need the reset done in BUS-select to set AnyDevices
+	if ( in->AnyDevices == anydevices_no ) {
+		ds->LastDevice = 1;
+		return search_done;
 	}
 
 	// build the command stream
@@ -584,18 +665,19 @@ static int DS2480_next_both(struct device_search *ds, const struct parsedname *p
 	// after last discrepancy so leave zeros
 
 	// flush the buffers
-	COM_flush(pn->selected_connection);
+	DS2480_flush(in);
 
 	// search ON
 	// change back to command mode
 	// send the packet
+	// cannot use single-bit mode with search accerator
 	// search OFF
-	if ((ret = BUS_send_data(&(ds->search), 1, pn))
-		|| (ret = DS2480_sendout_cmd(&searchon, 1, pn))
-		|| (ret = BUS_sendback_data(bitpairs, bitpairs, 16, pn))
-		|| (ret = DS2480_sendout_cmd(&searchoff, 1, pn))
+	if ( BAD(BUS_send_data(&(ds->search), 1, pn))
+		|| BAD(DS2480_sendout_cmd(&searchon, 1, in))
+		|| BAD( DS2480_sendback_data(bitpairs, bitpairs, 16, pn) )
+		|| BAD(DS2480_sendout_cmd(&searchoff, 1, in))
 		) {
-		return ret;
+		return search_error;
 	}
 
 	// interpret the bit stream
@@ -608,9 +690,14 @@ static int DS2480_next_both(struct device_search *ds, const struct parsedname *p
 		}
 	}
 
+	if ( sn[0]==0xFF && sn[1]==0xFF && sn[2]==0xFF && sn[3]==0xFF && sn[4]==0xFF && sn[5]==0xFF && sn[6]==0xFF && sn[7]==0xFF ) {
+		// special case for no alarm present
+		return search_done ;
+	}
+
 	// CRC check
 	if (CRC8(sn, 8) || (ds->LastDiscrepancy == 63) || (sn[0] == 0)) {
-		return -EIO;
+		return search_error;
 	}
 
 	// successful search
@@ -622,15 +709,11 @@ static int DS2480_next_both(struct device_search *ds, const struct parsedname *p
 	// copy the SerialNum to the buffer
 	memcpy(ds->sn, sn, 8);
 
-	if ((sn[0] & 0x7F) == DS2404_family) {
-		/* We found a DS1994/DS2404 which require longer delays */
-		pn->selected_connection->ds2404_compliance = 1;
-	}
 	// set the count
 	ds->LastDiscrepancy = mismatched;
 
-	LEVEL_DEBUG("SN found: " SNformat "\n", SNvar(ds->sn));
-	return 0;
+	LEVEL_DEBUG("SN found: " SNformat, SNvar(ds->sn));
+	return search_good;
 }
 
 //--------------------------------------------------------------------------
@@ -643,10 +726,11 @@ static int DS2480_next_both(struct device_search *ds, const struct parsedname *p
 /* Returns 0=good
    bad = -EIO
  */
-static int DS2480_PowerByte(const BYTE byte, BYTE * resp, const UINT delay, const struct parsedname *pn)
+static GOOD_OR_BAD DS2480_PowerByte(const BYTE byte, BYTE * resp, const UINT delay, const struct parsedname *pn)
 {
-	int ret;
-	BYTE bits = CMD_COMM | FUNCTSEL_BIT | DS2480b_speed_byte(pn) ;
+	GOOD_OR_BAD ret;
+	struct connection_in * in = pn->selected_connection ;
+	BYTE bits = CMD_COMM | FUNCTSEL_BIT | DS2480b_speed_byte(in) ;
 	BYTE cmd[] = {
 		// bit 1
 		((byte & 0x01) ? BITPOL_ONE : BITPOL_ZERO) | bits | PRIME5V_FALSE,
@@ -669,16 +753,16 @@ static int DS2480_PowerByte(const BYTE byte, BYTE * resp, const UINT delay, cons
 	BYTE response[1];
 
 	// flush the buffers
-	COM_flush(pn->selected_connection);
+	DS2480_flush(in);
 
 	// send the packet
 	// read back the 8 byte response from sending the 5V pulse
-	ret = DS2480_sendback_cmd(cmd, respbits, 8, pn);
+	ret = DS2480_sendback_cmd(cmd, respbits, 8, in);
 
 	UT_delay(delay);
 
 	// return to normal level
-	DS2480_stop_pulse(response, pn);
+	DS2480_stop_pulse(response, in);
 
 	resp[0] = ((respbits[7] & 1) << 7)
 		| ((respbits[6] & 1) << 6)
@@ -689,67 +773,136 @@ static int DS2480_PowerByte(const BYTE byte, BYTE * resp, const UINT delay, cons
 		| ((respbits[1] & 1) << 1)
 		| ((respbits[0] & 1));
 
-	return ret;
+	return ret ;
 }
 
-static int DS2480_stop_pulse(BYTE * response, const struct parsedname *pn)
+//--------------------------------------------------------------------------
+// Send 1 bit of communication to the 1-Wire Net and verify that the
+// bit read from the 1-Wire Net is the same (write operation).
+// Delay delay msec and return to normal
+//
+/* Returns 0=good
+   bad = -EIO
+ */
+static GOOD_OR_BAD DS2480_PowerBit(const BYTE byte, BYTE * resp, const UINT delay, const struct parsedname *pn)
+{
+	GOOD_OR_BAD ret;
+	struct connection_in * in = pn->selected_connection ;
+	BYTE bits = CMD_COMM | FUNCTSEL_BIT | DS2480b_speed_byte(in) ;
+	BYTE cmd[] = {
+		((byte & 0x01) ? BITPOL_ONE : BITPOL_ZERO) | bits | PRIME5V_TRUE,
+	};
+	BYTE respbits[1];
+	BYTE response[1];
+
+	// flush the buffers
+	DS2480_flush(in);
+
+	// send the packet
+	// read back the 1 byte response from sending the 5V pulse
+	ret = DS2480_sendback_cmd(cmd, respbits, 1, in);
+
+	UT_delay(delay);
+
+	// return to normal level
+	DS2480_stop_pulse(response, in);
+
+	resp[0] = (respbits[0] & 0x01);
+
+	return ret ;
+}
+
+//--------------------------------------------------------------------------
+// Send 1 bit of communication to the 1-Wire Net and verify that the
+// 8 bits read from the 1-Wire Net is the same (write operation).
+//
+/* Returns 0=good
+   bad = -EIO
+ */
+static GOOD_OR_BAD DS2480_sendback_bits(const BYTE * databits, BYTE * respbits, const size_t len, const struct parsedname * pn)
+{
+	struct connection_in * in = pn->selected_connection ;
+	BYTE bits = CMD_COMM | FUNCTSEL_BIT | DS2480b_speed_byte(in) | PRIME5V_FALSE;
+	size_t counter ;
+
+	for ( counter=0 ; counter < len ; ++counter ) {
+		BYTE cmd[] = { ((databits[counter] & 0x01) ? BITPOL_ONE : BITPOL_ZERO) | bits, };
+
+		// send the packet
+		// read back the 1 byte response from sending the bit
+		RETURN_BAD_IF_BAD( DS2480_sendback_cmd(cmd, &respbits[counter], 1, in) ) ;
+	}
+	return 0 ;
+}
+
+static void DS2480_flush( const struct connection_in * in )
+{
+	COM_flush( in ) ;
+}
+
+static void DS2480_slurp( struct connection_in * in )
+{
+	COM_slurp( in ) ;
+}
+
+static GOOD_OR_BAD DS2480_stop_pulse(BYTE * response, struct connection_in * in)
 {
 	BYTE cmd[1] = { MODE_STOP_PULSE, };
 	// read back the 8 byte response from setting time limit
-	return DS2480_sendback_cmd(cmd, response, 1, pn);
+	return DS2480_sendback_cmd(cmd, response, 1, in);
 }
 
 /* Send a 12v 480usec pulse on the 1wire bus to program the EPROM */
 // returns 0 if good
-static int DS2480_ProgramPulse(const struct parsedname *pn)
+static GOOD_OR_BAD DS2480_ProgramPulse(const struct parsedname *pn)
 {
-	int ret;
+	struct connection_in * in = pn->selected_connection ;
+	GOOD_OR_BAD ret;
 	BYTE cmd[] = { CMD_COMM | FUNCTSEL_CHMOD | BITPOL_12V | SPEEDSEL_PULSE, };
 	BYTE response_mask = 0xFC;
 	BYTE command_resp[1];
 	BYTE stop_pulse[1] ;
 
-	COM_flush(pn->selected_connection);
+	DS2480_flush(in);
 
 	// send the packet
 	// read back the 8 byte response from sending the 12V pilse
-	ret = DS2480_sendback_cmd(cmd, command_resp, 1, pn);
+	ret = DS2480_sendback_cmd(cmd, command_resp, 1, in);
 
-	if (ret == 0) {
+	if ( GOOD(ret) ) {
 		UT_delay_us(520);
 	}
 	// return to normal level
-	DS2480_stop_pulse(stop_pulse, pn);
+	DS2480_stop_pulse(stop_pulse, in);
 
-	if (ret) {
-		return ret;
-	}
+	RETURN_BAD_IF_BAD(ret) ;
 
 	if ((command_resp[0] & response_mask) != (cmd[0] & response_mask)) {
-		ret = -EIO;
+		return gbBAD;
 	}
 
-	return ret;
+	return gbGOOD;
 }
 
-//
-// Write a string to the serial port
-/* return 0=good,
-          -EIO = error
- */
-static int DS2480_write(const BYTE * buf, const size_t size, const struct parsedname *pn)
+// Write to the output -- works for tcp and COM
+static GOOD_OR_BAD DS2480_write(const BYTE * buf, const size_t size, struct connection_in * in)
 {
-	return COM_write( buf, size, pn->selected_connection );
+	switch( SOC(in)->type ) {
+		case ct_telnet:
+			return telnet_write_binary( buf, size, in) ;
+		case ct_serial:
+		default:
+			return COM_write( buf, size, in ) ;
+	}
 }
 
 /* Assymetric */
 /* Read from DS2480 with timeout on each character */
 // NOTE: from PDkit, read 1-byte at a time
 // NOTE: change timeout to 40msec from 10msec for LINK (emulation mode)
-// returns 0=good 1=bad
-static int DS2480_read(BYTE * buf, const size_t size, const struct parsedname *pn)
+static GOOD_OR_BAD DS2480_read(BYTE * buf, const size_t size, struct connection_in * in)
 {
-	return COM_read( buf, size, pn->selected_connection ) ;
+	return COM_read( buf, size, in ) ;
 }
 
 //
@@ -760,21 +913,21 @@ static int DS2480_read(BYTE * buf, const size_t size, const struct parsedname *p
 // The only uses for this alone are the single byte search accelerators
 // so size checking will be done in sendback
 // return 0=good
-static int DS2480_sendout_cmd(const BYTE * cmd, const size_t len, const struct parsedname *pn)
+static GOOD_OR_BAD DS2480_sendout_cmd(const BYTE * cmd, const size_t len, struct connection_in * in)
 {
-	if (pn->selected_connection->connin.serial.mode == ds2480b_command_mode ) {
+	if (in->master.serial.mode == ds2480b_command_mode ) {
 		// already in command mode -- very easy
-		return DS2480_write(cmd, (unsigned) len, pn);
+		return DS2480_write(cmd, (unsigned) len, in);
 	} else {
 		// need to switch to command mode
 		// add to current string for efficiency
 		BYTE cmd_plus[len+1] ;
 		cmd_plus[0] = MODE_COMMAND ;
 		memcpy( &cmd_plus[1], cmd, len ) ;
-		// MODE_COMMAND is one of the reservred commands that does not generate a response byte
+		// MODE_COMMAND is one of the reserved commands that does not generate a response byte
 		// page 6 of datasheet: http://datasheets.maxim-ic.com/en/ds/DS2480B.pdf
-		pn->selected_connection->connin.serial.mode = ds2480b_command_mode ;
-		return DS2480_write(cmd_plus, (unsigned) len+1, pn);
+		in->master.serial.mode = ds2480b_command_mode ;
+		return DS2480_write(cmd_plus, (unsigned) len+1, in);
 	}
 }
 
@@ -783,31 +936,23 @@ static int DS2480_sendout_cmd(const BYTE * cmd, const size_t len, const struct p
 //  Send a command and return response block
 //  puts into command mode if needed.
 // return 0=good
-static int DS2480_sendback_cmd(const BYTE * cmd, BYTE * resp, const size_t len, const struct parsedname *pn)
+static GOOD_OR_BAD DS2480_sendback_cmd(const BYTE * cmd, BYTE * resp, const size_t len, struct connection_in * in)
 {
 	size_t bytes_so_far = 0 ;
-	size_t extra_byte_for_modeshift  = (pn->selected_connection->connin.serial.mode != ds2480b_command_mode) ? 1 : 0 ;
+	size_t extra_byte_for_modeshift  = (in->master.serial.mode != ds2480b_command_mode) ? 1 : 0 ;
+
 
 	while ( bytes_so_far < len ) {
-		int ret;
 		size_t bytes_this_segment = len - bytes_so_far ;
 		if ( bytes_this_segment > MAX_SEND_SIZE - extra_byte_for_modeshift ) {
 			bytes_this_segment = MAX_SEND_SIZE - extra_byte_for_modeshift ;
 			extra_byte_for_modeshift = 0 ;
 		}
-		ret = DS2480_sendout_cmd( &cmd[bytes_so_far], bytes_this_segment, pn);
-		if ( ret ) {
-            LEVEL_DEBUG("write error\n") ;
-			return ret ;
-		}
-		ret = DS2480_read(       &resp[bytes_so_far], bytes_this_segment, pn);
-		if ( ret ) {
-            LEVEL_DEBUG("read error\n") ;
-            return ret ;
-		}
+		RETURN_BAD_IF_BAD( DS2480_sendout_cmd( &cmd[bytes_so_far], bytes_this_segment, in) );
+		RETURN_BAD_IF_BAD( DS2480_read( &resp[bytes_so_far], bytes_this_segment, in) );
 		bytes_so_far += bytes_this_segment ;
 	}
-	return 0 ;
+	return gbGOOD ;
 }
 
 //
@@ -816,8 +961,9 @@ static int DS2480_sendback_cmd(const BYTE * cmd, BYTE * resp, const size_t len,
 //  puts into data mode if needed.
 // Repeat magic MODE_COMMAND byte to show true data
 // return 0=good
-static int DS2480_sendback_data(const BYTE * data, BYTE * resp, const size_t len, const struct parsedname *pn)
+static GOOD_OR_BAD DS2480_sendback_data(const BYTE * data, BYTE * resp, const size_t len, const struct parsedname *pn)
 {
+	struct connection_in * in = pn->selected_connection ;
 	BYTE sendout[MAX_SEND_SIZE] ;
 	size_t bytes_this_segment = 0 ;
 	size_t bytes_at_segment_start = 0 ;
@@ -826,20 +972,19 @@ static int DS2480_sendback_data(const BYTE * data, BYTE * resp, const size_t len
 	// skip if nothing
 	// don't even switch to data mode
 	if ( len == 0 ) {
-		return 0 ;
+		return gbGOOD ;
 	}
 
 	// switch mode if needed
-	if (pn->selected_connection->connin.serial.mode != ds2480b_data_mode) {
+	if (in->master.serial.mode != ds2480b_data_mode) {
 		// this is one of the "reserved commands" that does not generate a resonse byte
 		// page 6 of the datasheet http://datasheets.maxim-ic.com/en/ds/DS2480B.pdf
 		sendout[bytes_this_segment++] = MODE_DATA;
 		// change flag to data mode
-		pn->selected_connection->connin.serial.mode = ds2480b_data_mode;
+		in->master.serial.mode = ds2480b_data_mode;
 	}
 
-	while ( bytes_from_list < len )
-	{
+	while ( bytes_from_list < len ) {
 		BYTE current_char = data[bytes_from_list++] ;
 		// transfer this byte
 		sendout[ bytes_this_segment++ ] = current_char ;
@@ -852,22 +997,23 @@ static int DS2480_sendback_data(const BYTE * data, BYTE * resp, const size_t len
 		// need room for potentially 2 characters
 		if ( bytes_this_segment > MAX_SEND_SIZE-2 || bytes_from_list == len ) {
 			// write out
-			int ret = DS2480_write(sendout, bytes_this_segment, pn ) ;
-			if ( ret ) {
-				return ret ;
-			}
+			RETURN_BAD_IF_BAD( DS2480_write(sendout, bytes_this_segment, in ) ) ;
 
 			// read in
 			// note that read and write sizes won't match because of doubled bytes and mode shift
-			ret = DS2480_read(&resp[bytes_at_segment_start], bytes_from_list - bytes_at_segment_start, pn);
-			if ( ret ) {
-				return ret ;
-			}
+			RETURN_BAD_IF_BAD( DS2480_read(&resp[bytes_at_segment_start], bytes_from_list - bytes_at_segment_start, in) ) ;
 
 			// move indexes for next segment
 			bytes_at_segment_start = bytes_from_list ;
 			bytes_this_segment = 0 ;
 		}
 	}
-	return 0 ;
+	return gbGOOD ;
 }
+
+static void DS2480_close(struct connection_in *in)
+{
+	// the standard COM_free cleans up the connection
+	(void) in ;
+}
+
