@@ -1,5 +1,5 @@
 /*
-$Id: ow_lib_close.c,v 1.9 2009/03/28 19:00:34 alfille Exp $
+$Id: ow_lib_close.c,v 1.20 2010/11/16 17:11:30 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
@@ -18,44 +18,26 @@ $Id: ow_lib_close.c,v 1.9 2009/03/28 19:00:34 alfille Exp $
 /* All ow library closeup */
 void LibClose(void)
 {
-	LEVEL_CALL("Starting Library cleanup\n");
+	LEVEL_CALL("Starting Library cleanup");
 	LibStop();
 	PIDstop();
 	DeviceDestroy();
 
-#if OW_MT
-	if (Mutex.pmattr) {
-		my_pthread_mutexattr_destroy(Mutex.pmattr);
-		Mutex.pmattr = NULL;
-	}
-#endif							/* OW_MT */
+	_MUTEX_ATTR_DESTROY(Mutex.mattr);
 
-	if (Globals.announce_name) {
-		owfree(Globals.announce_name);
-		Globals.announce_name = NULL;
-	}
-
-	if (Globals.progname) {
-		owfree(Globals.progname);
-		Globals.progname = NULL;
-	}
 #if OW_ZERO
-	if (Globals.browse && (libdnssd != NULL)) {
-		DNSServiceRefDeallocate(Globals.browse);
-	}
-
+	// Used by browse and announce
 	OW_Free_dnssd_library();
 	OW_Free_avahi_library();
 #endif
 
-	if (Globals.fatal_debug_file) {
-		owfree(Globals.fatal_debug_file);
-		Globals.fatal_debug_file = NULL;
-	}
-
-	LEVEL_CALL("Finished Library cleanup\n");
+	LEVEL_CALL("Finished Library cleanup");
 	if (log_available) {
 		closelog();
 		log_available = 0;
 	}
+	SAFEFREE(Globals.announce_name) ;
+	SAFEFREE(Globals.progname) ;
+	SAFEFREE(Globals.fatal_debug_file) ;
+	LEVEL_DEBUG("Libraries closed");
 }
