@@ -1,5 +1,5 @@
 /*
-$Id: ow_lib_stop.c,v 1.5 2008/11/07 02:51:15 alfille Exp $
+$Id: ow_lib_stop.c,v 1.8 2010/01/25 16:08:27 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
@@ -20,14 +20,12 @@ $Id: ow_lib_stop.c,v 1.5 2008/11/07 02:51:15 alfille Exp $
 void LibStop(void)
 {
 	char *argv[1] = { NULL };
-#if OW_CACHE
-	LEVEL_CALL("Clear Cache\n");
+	LEVEL_CALL("Clear Cache");
 	Cache_Clear();
-#endif							/* OW_CACHE */
-	LEVEL_CALL("Closing input devices\n");
+	LEVEL_CALL("Closing input devices");
 	FreeInAll();
-	LEVEL_CALL("Closing outout devices\n");
-	FreeOut();
+	LEVEL_CALL("Closing outout devices");
+	FreeOutAll();
 
 
 	/* Have to reset more internal variables, and this should be fixed
