@@ -1,5 +1,5 @@
 /*
-$Id: ow_parse_sn.c,v 1.1 2009/04/12 03:40:16 alfille Exp $
+$Id: ow_parse_sn.c,v 1.6 2010/05/05 02:15:26 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
@@ -13,7 +13,8 @@ $Id: ow_parse_sn.c,v 1.1 2009/04/12 03:40:16 alfille Exp $
 #include "owfs_config.h"
 #include "ow.h"
 
-int Parse_SerialNumber(char *sn_char, BYTE * sn){
+enum parse_serialnumber Parse_SerialNumber(char *sn_char, BYTE * sn)
+{
 	ASCII ID[14];
 	int i;
 
@@ -24,7 +25,7 @@ int Parse_SerialNumber(char *sn_char, BYTE * sn){
 		if (isxdigit(*sn_char)) {
 			ID[i] = *sn_char;
 		} else {
-			return 1; // non-hex
+			return sn_alias; // non-hex
 		}
 	}
 	sn[0] = string2num(&ID[0]);
@@ -34,19 +35,19 @@ int Parse_SerialNumber(char *sn_char, BYTE * sn){
 	sn[4] = string2num(&ID[8]);
 	sn[5] = string2num(&ID[10]);
 	sn[6] = string2num(&ID[12]);
-	sn[7] = CRC8compute(sn, 7, 0);
+	sn[7] = CRC8compute(sn, SERIAL_NUMBER_SIZE-1, 0);
 	if (*sn_char == '.') {
 		++sn_char;
 	}
 	// Check CRC8 if given
 	if (isxdigit(sn_char[0]) && isxdigit(sn_char[1])) {
 		char crc[2];
-		num2string(crc, sn[7]);
+		num2string(crc, sn[SERIAL_NUMBER_SIZE-1]);
 		if (strncasecmp(crc, sn_char, 2)) {
-			return 1;
+			return sn_invalid;
 		}
+		sn_char += 2 ;
 	}
-
-	return 0;
+	return ( *sn_char == '\0' ) ? sn_valid : sn_alias ;
 }
 
