@@ -1,5 +1,5 @@
 /*
-$Id: ow_parsename.c,v 1.166 2009/04/20 01:29:31 alfille Exp $
+$Id: ow_parsename.c,v 1.203 2011/06/23 01:21:20 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
@@ -14,18 +14,7 @@ $Id: ow_parsename.c,v 1.166 2009/04/20 01:29:31 alfille Exp $
 #include "ow_devices.h"
 #include "ow_counters.h"
 
-static int BranchAdd(struct parsedname *pn);
-
-enum parse_enum {
-	parse_first,
-	parse_done,
-	parse_error,
-	parse_real,
-	parse_branch,
-	parse_nonreal,
-	parse_prop,
-	parse_subprop
-};
+static ZERO_OR_ERROR BranchAdd(struct parsedname *pn);
 
 enum parse_pass {
 	parse_pass_pre_remote,
@@ -39,17 +28,23 @@ struct parsedname_pointers {
 	char *pathlast;
 };
 
-
+static enum parse_enum set_type( enum ePN_type epntype, struct parsedname * pn ) ;
 static enum parse_enum Parse_Unspecified(char *pathnow, enum parse_pass remote_status, struct parsedname *pn);
 static enum parse_enum Parse_Branch(char *pathnow, enum parse_pass remote_status, struct parsedname *pn);
 static enum parse_enum Parse_Real(char *pathnow, enum parse_pass remote_status, struct parsedname *pn);
 static enum parse_enum Parse_NonReal(char *pathnow, struct parsedname *pn);
 static enum parse_enum Parse_RealDevice(char *filename, enum parse_pass remote_status, struct parsedname *pn);
-static enum parse_enum Parse_NonRealDevice(char *filename, struct parsedname *pn);
 static enum parse_enum Parse_Property(char *filename, struct parsedname *pn);
+
+static enum parse_enum Parse_RealDeviceSN(enum parse_pass remote_status, struct parsedname *pn);
+static enum parse_enum Parse_NonRealDevice(char *filename, struct parsedname *pn);
 static enum parse_enum Parse_Bus(char *pathnow, struct parsedname *pn);
-static int FS_ParsedName_anywhere(const char *path, enum parse_pass remote_status, struct parsedname *pn);
-static int FS_ParsedName_setup(struct parsedname_pointers *pp, const char *path, struct parsedname *pn);
+static enum parse_enum Parse_Alias(char *filename, enum parse_pass remote_status, struct parsedname *pn);
+static enum parse_enum Parse_Alias_Known( char *filename, enum parse_pass remote_status, struct parsedname *pn);
+static void ReplaceAliasInPath( char * filename, struct parsedname * pn);
+
+static ZERO_OR_ERROR FS_ParsedName_anywhere(const char *path, enum parse_pass remote_status, struct parsedname *pn);
+static ZERO_OR_ERROR FS_ParsedName_setup(struct parsedname_pointers *pp, const char *path, struct parsedname *pn);
 
 #define BRANCH_INCR (9)
 
@@ -61,96 +56,104 @@ void FS_ParsedName_destroy(struct parsedname *pn)
 	if (!pn) {
 		return;
 	}
-	LEVEL_DEBUG("%s\n", SAFESTRING(pn->path));
+	LEVEL_DEBUG("%s", SAFESTRING(pn->path));
 	CONNIN_RUNLOCK ;
-	//printf("PNDestroy bp (%d)\n",BusIsServer(pn->selected_connection)) ;
-	if (pn->bp) {
-		owfree(pn->bp);
-		pn->bp = NULL;
-	}
-	//printf("PNDestroy path (%d)\n",BusIsServer(pn->selected_connection)) ;
-	if (pn->path) {
-		owfree(pn->path);
-		pn->path = NULL;
-	}
-	//printf("PNDestroy lock (%d)\n",BusIsServer(pn->selected_connection)) ;
-	if (pn->lock) {
-		owfree(pn->lock);
-		pn->lock = NULL;
-	}
-	//printf("PNDestroy done (%d)\n",BusIsServer(pn->selected_connection)) ;
-	// Tokenstring is part of a larger allocation and destroyed separately
+	SAFEFREE(pn->bp) ;
+	SAFEFREE(pn->path) ;
 }
 
+// Path is either NULL (in which case a minimal structure is created that doesn't need Destroy -- used for Bus Master setups)
+// Or Path is a full "filename" type string of form: 10.1243Ab000 or uncached/bus.0/statistics etc.
+// The Path passed in isn't altered, but 2 copies are made -- one with the full path, the other (to_server) has the first bus.n removed.
+// An initial / is added to the path, and the full length has to be less than MAX_PATH (2048)
+// For efficiency, the two path copies are allocated in the same call, and so can be removed together.
+
 /* Parse a path to check it's validity and attach to the propery data structures */
-int FS_ParsedName(const char *path, struct parsedname *pn)
+ZERO_OR_ERROR FS_ParsedName(const char *path, struct parsedname *pn)
 {
 	return FS_ParsedName_anywhere(path, parse_pass_pre_remote, pn);
 }
 
 /* Parse a path from a remote source back -- so don't check presence */
-int FS_ParsedName_BackFromRemote(const char *path, struct parsedname *pn)
+ZERO_OR_ERROR FS_ParsedName_BackFromRemote(const char *path, struct parsedname *pn)
 {
 	return FS_ParsedName_anywhere(path, parse_pass_post_remote, pn);
 }
 
 /* Parse off starting "mode" directory (uncached, alarm...) */
-static int FS_ParsedName_anywhere(const char *path, enum parse_pass remote_status, struct parsedname *pn)
+static ZERO_OR_ERROR FS_ParsedName_anywhere(const char *path, enum parse_pass remote_status, struct parsedname *pn)
 {
 	struct parsedname_pointers s_pp;
 	struct parsedname_pointers *pp = &s_pp;
-	int ret = 0;
+	ZERO_OR_ERROR parse_error_status = 0;
 	enum parse_enum pe = parse_first;
 
 	// To make the debug output useful it's cleared here.
 	// Even on normal glibc, errno isn't cleared on good system calls
 	errno = 0;
 
-	LEVEL_CALL("path=[%s]\n", SAFESTRING(path));
+	LEVEL_CALL("path=[%s]", SAFESTRING(path));
 
-	ret = FS_ParsedName_setup(pp, path, pn);
-	if (ret) {
-		return ret;
+	parse_error_status = FS_ParsedName_setup(pp, path, pn);
+	if (parse_error_status) {
+		return parse_error_status;
 	}
-	//printf("1pathnow=[%s] pathnext=[%s] pn->type=%d\n", pathnow, pathnext, pn->type);
-	if (path == NULL) {
+	if (path == NO_PATH) {
 		return 0;
 	}
 
 	while (1) {
-		//printf("PARSENAME parse_enum=%d pathnow=%s\n",pe,SAFESTRING(pp->pathnow));
 		// Check for extreme conditions (done, error)
 		switch (pe) {
 
 		case parse_done:		// the only exit!
-			//LEVEL_DEBUG("PARSENAME parse_done\n") ;
-			//printf("PARSENAME end ret=%d\n",ret) ;
-			if (ret) {
+			//LEVEL_DEBUG("PARSENAME parse_done") ;
+			//printf("PARSENAME end parse_error_status=%d\n",parse_error_status) ;
+			if (parse_error_status) {
 				FS_ParsedName_destroy(pn);
-			} else {
-				//printf("%s: Parse %s before corrections: %.4X -- state = %d\n",(back_from_remote)?"BACK":"FORE",pn->path,pn->state,pn->type) ;
-				if (SpecifiedRemoteBus(pn) && !SpecifiedVeryRemoteBus(pn)) {
-					// promote interface of remote bus to local
-					if (pn->type == ePN_interface) {
-						pn->state &= ~ePS_busremote;
-						pn->state |= ePS_buslocal;
+				return parse_error_status ;
+			}
 
-						// non-root remote bus
-					} else if (pn->type != ePN_root) {
+			if ( pp->pathnext != NULL ) {
+				// extra text -- make this an error
+				LEVEL_DEBUG("Extra text in file %s",pp->pathnext) ;
+				parse_error_status = -ENOENT;
+				pe = parse_done;
+				continue;				
+			}
+			
+			//printf("%s: Parse %s before corrections: %.4X -- state = %d\n",(back_from_remote)?"BACK":"FORE",pn->path,pn->state,pn->type) ;
+			// Play with remote levels
+			switch ( pn->type ) {
+				case ePN_interface:
+					if ( SpecifiedVeryRemoteBus(pn) ) {
+						// veryremote -> remote
+						pn->state &= ~ePS_busveryremote ;
+					} else if ( SpecifiedRemoteBus(pn) ) {
+						// remote -> local
+						pn->state &= ~ePS_busremote ;
+						pn->state |= ePS_buslocal ;
+					}
+					break ;
+					
+				case ePN_root:
+					// root buses are considered "real"
+					pn->type = ePN_real;	// default state
+					break ;
+				default:
+					// everything else gets promoted so directories aren't added on
+					if ( SpecifiedRemoteBus(pn) ) {
+						// very remote
 						pn->state |= ePS_busveryremote;
 					}
-				}
-				// root buses are considered "real"
-				if (pn->type == ePN_root) {
-					pn->type = ePN_real;	// default state
-				}
+					break ;
 			}
 			//printf("%s: Parse %s after  corrections: %.4X -- state = %d\n\n",(back_from_remote)?"BACK":"FORE",pn->path,pn->state,pn->type) ;
-			return ret;
+			return 0;
 
 		case parse_error:
-			//LEVEL_DEBUG("PARSENAME parse_error\n") ;
-			ret = -ENOENT;
+			//LEVEL_DEBUG("PARSENAME parse_error") ;
+			parse_error_status = -ENOENT;
 			pe = parse_done;
 			continue;
 
@@ -159,8 +162,8 @@ static int FS_ParsedName_anywhere(const char *path, enum parse_pass remote_statu
 		}
 
 		// break out next name in path, make sure pp->pathnext isn't NULL. (SIGSEGV in uClibc)
-		pp->pathnow = (pp->pathnext) ? strsep(&(pp->pathnext), "/") : NULL;
-		//LEVEL_DEBUG("PARSENAME pathnow=[%s] rest=[%s]\n",pathnow,pathnext) ;
+		pp->pathnow = (pp->pathnext != NULL) ? strsep(&(pp->pathnext), "/") : NULL;
+		//LEVEL_DEBUG("PARSENAME pathnow=[%s] rest=[%s]",pp->pathnow,pp->pathnext) ;
 		if (pp->pathnow == NULL || pp->pathnow[0] == '\0') {
 			pe = parse_done;
 			continue;
@@ -169,17 +172,17 @@ static int FS_ParsedName_anywhere(const char *path, enum parse_pass remote_statu
 		switch (pe) {
 
 		case parse_first:
-			//LEVEL_DEBUG("PARSENAME parse_first\n") ;
+			//LEVEL_DEBUG("PARSENAME parse_first") ;
 			pe = Parse_Unspecified(pp->pathnow, remote_status, pn);
 			break;
 
 		case parse_real:
-			//LEVEL_DEBUG("PARSENAME parse_real\n") ;
+			//LEVEL_DEBUG("PARSENAME parse_real") ;
 			pe = Parse_Real(pp->pathnow, remote_status, pn);
 			break;
 
 		case parse_branch:
-			//LEVEL_DEBUG("PARSENAME parse_branch\n") ;
+			//LEVEL_DEBUG("PARSENAME parse_branch") ;
 			pe = Parse_Branch(pp->pathnow, remote_status, pn);
 			break;
 
@@ -189,15 +192,16 @@ static int FS_ParsedName_anywhere(const char *path, enum parse_pass remote_statu
 			break;
 
 		case parse_prop:
-			//LEVEL_DEBUG("PARSENAME parse_prop\n") ;
-			pn->dirlength = pp->pathnow - pp->pathcpy ;
+			//LEVEL_DEBUG("PARSENAME parse_prop") ;
+			pn->dirlength = pp->pathnow - pp->pathcpy + 1 ;
+			//LEVEL_DEBUG("Dirlength=%d <%*s>",pn->dirlength,pn->dirlength,pn->path) ;
 			//printf("dirlength = %d which makes the path <%s> <%.*s>\n",pn->dirlength,pn->path,pn->dirlength,pn->path);
 			pp->pathlast = pp->pathnow;	/* Save for concatination if subdirectory later wanted */
 			pe = Parse_Property(pp->pathnow, pn);
 			break;
 
 		case parse_subprop:
-			//LEVEL_DEBUG("PARSENAME parse_subprop\n") ;
+			//LEVEL_DEBUG("PARSENAME parse_subprop") ;
 			pp->pathnow[-1] = '/';
 			pe = Parse_Property(pp->pathlast, pn);
 			break;
@@ -212,9 +216,9 @@ static int FS_ParsedName_anywhere(const char *path, enum parse_pass remote_statu
 }
 
 /* Initial memory allocation and pn setup */
-static int FS_ParsedName_setup(struct parsedname_pointers *pp, const char *path, struct parsedname *pn)
+static ZERO_OR_ERROR FS_ParsedName_setup(struct parsedname_pointers *pp, const char *path, struct parsedname *pn)
 {
-	if (pn == NULL) {
+	if (pn == NO_PARSEDNAME) {
 		return -EINVAL;
 	}
 
@@ -222,24 +226,43 @@ static int FS_ParsedName_setup(struct parsedname_pointers *pp, const char *path,
 	pn->known_bus = NULL;		/* all buses */
 
 	/* Set the persistent state info (temp scale, ...) -- will be overwritten by client settings in the server */
-	SGLOCK;
-	pn->sg = SemiGlobal | SHOULD_RETURN_BUS_LIST;	// initial flag as the bus-returning level, will change if a bus is specified
-	SGUNLOCK;
+	CONTROLFLAGSLOCK;
+	pn->control_flags = LocalControlFlags | SHOULD_RETURN_BUS_LIST;	// initial flag as the bus-returning level, will change if a bus is specified
+	CONTROLFLAGSUNLOCK;
 
 	// initialization
-	pp->pathnow = NULL;
-	pp->pathlast = NULL;
-	pp->pathnext = NULL;
+	pp->pathnow = NO_PATH;
+	pp->pathlast = NO_PATH;
+	pp->pathnext = NO_PATH;
 
 	/* Default attributes */
 	pn->state = ePS_normal;
 	pn->type = ePN_root;
 
+	/* uncached attribute */
+	if ( Globals.uncached ) {
+		// local settings (--uncached) can set
+		pn->state |= ePS_uncached;
+	}
+	// Also can be set by path ("/uncached")
+	// Also in owserver, can be set by client flags
+	// sibling inherits parent
+	
+	/* unaliased attribute */
+	if ( Globals.unaliased ) {
+		// local settings (--unalaised) can set
+		pn->state |= ePS_unaliased;
+	}
+	// Also can be set by path ("/unaliased")
+	// Also in owserver, can be set by client flags
+	// sibling inherits parent
+
 	/* No device lock yet assigned */
 	pn->lock = NULL ;
 
-	/* minimal structure for initial bus "detect" use -- really has connection and SemiGlobal only */
-	if (path == NULL) {
+	/* minimal structure for initial bus "detect" use -- really has connection and LocalControlFlags only */
+	pn->dirlength = -1 ;
+	if (path == NO_PATH) {
 		return 0;
 	}
 
@@ -247,44 +270,55 @@ static int FS_ParsedName_setup(struct parsedname_pointers *pp, const char *path,
 		return -EBADF;
 	}
 
-	pn->path = (char *) owmalloc(2 * strlen(path) + 2);
-	if (pn->path == NULL) {
+	pn->path = (char *) owmalloc(2 * PATH_MAX + 4);
+	if (pn->path == NO_PATH) {
 		return -ENOMEM;
 	}
 
-	/* make a copy for destructive parsing */
-	strcpy(pp->pathcpy,path);
+	/* Have to save pn->path at once */
+	strcpy(pn->path, "/"); // initial slash
+	strcpy(pn->path+1, path[0]=='/'?path+1:path);
+	pn->path_to_server = pn->path + strlen(pn->path) + 1;
+	strcpy(pn->path_to_server, pn->path);
+
+	/* make a copy for destructive parsing  without initial '/'*/
+	strcpy(pp->pathcpy,&pn->path[1]);
+	/* pointer to rest of path after current token peeled off */
+	pp->pathnext = pp->pathcpy;
+	pn->dirlength = strlen(pn->path) ;
 
 	/* connection_in list and start */
 	/* ---------------------------- */
 	/* -- This is important:     -- */
-	/* -- But buses can --          */
+	/* --     Buses can --          */
 	/* -- be added by Browse so  -- */
 	/* -- a reader/writer lock is - */
 	/* -- held until ParsedNameDestroy */
 	/* ---------------------------- */
 
 	CONNIN_RLOCK;
-	pn->selected_connection = Inbound_Control.head ; // Default bus assignment
-
-	/* Have to save pn->path at once */
-	strcpy(pn->path, path);
-	pn->path_busless = pn->path + strlen(path) + 1;
-	strcpy(pn->path_busless, path);
-
-	/* pointer to rest of path after current token peeled off */
-	pp->pathnext = pp->pathcpy;
-
-	/* remove initial "/" */
-	if (pp->pathnext[0] == '/') {
-		++pp->pathnext;
-	}
-
-	pn->dirlength = strlen(path) ;
+	pn->selected_connection = NO_CONNECTION ; // Default bus assignment
 
 	return 0;
 }
 
+/* Used for virtual directories like settings and statistics
+ * If local, applies to all local (this program) and not a
+ * specific local bus.
+ * If remote, pass it on for the remote to handle
+ * */
+static enum parse_enum set_type( enum ePN_type epntype, struct parsedname * pn )
+{
+	if (SpecifiedLocalBus(pn)) {
+		return parse_error;
+	} else if ( ! SpecifiedRemoteBus(pn) ) {
+		pn->type |= ePS_busanylocal;
+	}
+	pn->type = epntype;
+	return parse_nonreal;
+}
+	
+
 // Early parsing -- only bus entries, uncached and text may have preceeded
 static enum parse_enum Parse_Unspecified(char *pathnow, enum parse_pass remote_status, struct parsedname *pn)
 {
@@ -292,32 +326,16 @@ static enum parse_enum Parse_Unspecified(char *pathnow, enum parse_pass remote_s
 		return Parse_Bus(pathnow, pn);
 
 	} else if (strcasecmp(pathnow, "settings") == 0) {
-		if (SpecifiedLocalBus(pn)) {
-			return parse_error;
-		}
-		pn->type = ePN_settings;
-		return parse_nonreal;
+		return set_type( ePN_settings, pn ) ;
 
 	} else if (strcasecmp(pathnow, "statistics") == 0) {
-		if (SpecifiedLocalBus(pn)) {
-			return parse_error;
-		}
-		pn->type = ePN_statistics;
-		return parse_nonreal;
+		return set_type( ePN_statistics, pn ) ;
 
 	} else if (strcasecmp(pathnow, "structure") == 0) {
-		if (SpecifiedLocalBus(pn)) {
-			return parse_error;
-		}
-		pn->type = ePN_structure;
-		return parse_nonreal;
+		return set_type( ePN_structure, pn ) ;
 
 	} else if (strcasecmp(pathnow, "system") == 0) {
-		if (SpecifiedLocalBus(pn)) {
-			return parse_error;
-		}
-		pn->type = ePN_system;
-		return parse_nonreal;
+		return set_type( ePN_system, pn ) ;
 
 	} else if (strcasecmp(pathnow, "interface") == 0) {
 		if (!SpecifiedBus(pn)) {
@@ -334,6 +352,10 @@ static enum parse_enum Parse_Unspecified(char *pathnow, enum parse_pass remote_s
 		pn->state |= ePS_uncached;
 		return parse_first;
 
+	} else if (strcasecmp(pathnow, "unaliased") == 0) {
+		pn->state |= ePS_unaliased;
+		return parse_first;
+
 	}
 
 	pn->type = ePN_real;
@@ -353,6 +375,7 @@ static enum parse_enum Parse_Branch(char *pathnow, enum parse_pass remote_status
 static enum parse_enum Parse_Real(char *pathnow, enum parse_pass remote_status, struct parsedname *pn)
 {
 	if (strcasecmp(pathnow, "simultaneous") == 0) {
+		LEVEL_DEBUG("TEST set simultaneous");
 		pn->selected_device = DeviceSimultaneous;
 		return parse_prop;
 
@@ -368,11 +391,13 @@ static enum parse_enum Parse_Real(char *pathnow, enum parse_pass remote_status,
 		pn->state |= ePS_uncached;
 		return parse_real;
 
+	} else if (strcasecmp(pathnow, "unaliased") == 0) {
+		pn->state |= ePS_unaliased;
+		return parse_real;
+
 	} else {
 		return Parse_RealDevice(pathnow, remote_status, pn);
 	}
-
-	return parse_error;
 }
 
 static enum parse_enum Parse_NonReal(char *pathnow, struct parsedname *pn)
@@ -385,6 +410,10 @@ static enum parse_enum Parse_NonReal(char *pathnow, struct parsedname *pn)
 		pn->state |= ePS_uncached;
 		return parse_nonreal;
 
+	} else if (strcasecmp(pathnow, "unaliased") == 0) {
+		pn->state |= ePS_unaliased;
+		return parse_nonreal;
+
 	} else {
 		return Parse_NonRealDevice(pathnow, pn);
 	}
@@ -396,14 +425,14 @@ static enum parse_enum Parse_NonReal(char *pathnow, struct parsedname *pn)
 static enum parse_enum Parse_Bus(char *pathnow, struct parsedname *pn)
 {
 	char *found;
-	int bus_number;
+	INDEX_OR_ERROR bus_number;
 	/* Processing for bus.X directories -- eventually will make this more generic */
 	if (!isdigit(pathnow[4])) {
 		return parse_error;
 	}
 
 	bus_number = atoi(&pathnow[4]);
-	if (bus_number < 0) {
+	if ( INDEX_NOT_VALID(bus_number) ) {
 		return parse_error;
 	}
 
@@ -419,42 +448,122 @@ static enum parse_enum Parse_Bus(char *pathnow, struct parsedname *pn)
 		return parse_first;
 	}
 
-	/* on return from remote directory ow_server.c:ServerDir
-	   SetKnownBus will be be performed elsewhere since the sending bus number is used */
-	/* this will only be reached once, because a local bus.x triggers "SpecifiedBus" */
-	//printf("SPECIFIED BUS for ParsedName PRE (%d):\n\tpath=%s\n\tpath_busless=%s\n\tKnnownBus=%d\tSpecifiedBus=%d\n",bus_number,   SAFESTRING(pn->path),SAFESTRING(pn->path_busless),KnownBus(pn),SpecifiedBus(pn));
-	if (bus_number < 0) {
-		return parse_error;
-	}
-
 	/* Since we are going to use a specific in-device now, set
 	 * pn->selected_connection to point at that device at once. */
-	SetSpecifiedBus(bus_number, pn);
+	if ( SetKnownBus(bus_number, pn) ) {
+		return parse_error ; // bus doesn't exist
+	}
+	pn->state |= BusIsServer((pn)->selected_connection) ? ePS_busremote : ePS_buslocal ;
 
 	if (SpecifiedLocalBus(pn)) {
 		/* don't return bus-list for local paths. */
-		pn->sg &= (~SHOULD_RETURN_BUS_LIST);
+		pn->control_flags &= (~SHOULD_RETURN_BUS_LIST);
 	}
 
-	/* Create the path without the "bus.x" part in pn->path_busless */
-	if (!(found = strstr(pn->path, "/bus."))) {	// no bus
-		int length = pn->path_busless - pn->path - 1;
-		strncpy(pn->path_busless, pn->path, length);	// just copy path
-	} else {
+	/* Create the path without the "bus.x" part in pn->path_to_server */
+	if ( (found = strstr(pn->path, "/bus.")) ) {
 		int length = found - pn->path;
-		strncpy(pn->path_busless, pn->path, length);
 		if ((found = strchr(found + 1, '/'))) {	// more after bus
-			strcpy(&(pn->path_busless[length]), found);	// copy rest
-
+			strcpy(&(pn->path_to_server[length]), found);	// copy rest
 		} else {
-			pn->path_busless[length] = '\0';	// add final null
+			pn->path_to_server[length] = '\0';	// add final null
 		}
 	}
-	//printf("SPECIFIED BUS for ParsedName POST (%d):\n\tpath=%s\n\tpath_busless=%s\n\tKnnownBus=%d\tSpecifiedBus=%d\n",bus_number,SAFESTRING(pn->path),SAFESTRING(pn->path_busless),KnownBus(pn),SpecifiedBus(pn));
-	//LEVEL_DEBUG("PARSENAME test path=%s, path_busless=%s\n",pn->path, pn->path_busless ) ;
 	return parse_first;
 }
 
+/* replace alias with sn */
+static void ReplaceAliasInPath( char * filename, struct parsedname * pn)
+{
+	int alias_len = strlen(filename) ;
+
+	char alias[alias_len + 2] ;
+
+	char * alias_loc ;
+	char * post_alias_loc ;
+
+	// check total length
+	if ( strlen(pn->path_to_server) + 14 - alias_len > PATH_MAX ) {
+		return ;
+	}
+	
+	strcpy( alias, "/") ;
+	strcat( alias, filename ) ; // we include an initial / to make the match more accurate
+
+	for ( alias_loc = pn->path_to_server; alias_loc != NULL ; alias_loc = strstr( alias_loc, alias ) ) {
+		++alias_loc ; // point after '/'
+
+		post_alias_loc = alias_loc + alias_len ;
+		switch ( post_alias_loc[0] ) {
+			case '\0':
+			case '/':
+				// move rest of path
+				memmove( &alias_loc[14], post_alias_loc, strlen(post_alias_loc)+1 ) ;
+				//write in serial number for alias
+				bytes2string( alias_loc, pn->sn, 7 ) ;
+				return ;
+			default:
+				// alias not really found (only partial match)
+				// loop starting one char later
+				break ;
+		}
+	}
+}
+
+/* This is when the alias name in mapped to a known serial number
+ * behaves much more like the standard handling -- bus from sn */
+static enum parse_enum Parse_Alias_Known( char *filename, enum parse_pass remote_status, struct parsedname *pn)
+{
+	if (remote_status == parse_pass_pre_remote) {
+		ReplaceAliasInPath( filename, pn ) ;
+	}
+
+	return Parse_RealDeviceSN( remote_status, pn ) ;
+}
+
+/* Get a device that isn't a serial number -- see if it's an alias */
+static enum parse_enum Parse_Alias(char *filename, enum parse_pass remote_status, struct parsedname *pn)
+{
+	INDEX_OR_ERROR bus ;
+	
+	// See if the alias is known in the permanent list. We get the serial number 
+	if ( GOOD( Cache_Get_Alias_SN(filename,pn->sn)) ) {
+		// Success! The alias is already registered and the serial
+		//  number just now loaded in pn->sn
+		return Parse_Alias_Known( filename, remote_status, pn ) ;
+	}
+
+	// By definition this is a remote device, or non-existent.
+	pn->selected_device = &RemoteDevice ;
+
+	// is alias name cached from previous query?
+	bus = Cache_Get_Alias_Bus( filename ) ;
+	if ( bus != INDEX_BAD ) {
+		// This alias is cached in temporary list
+		SetKnownBus(bus, pn);
+		return parse_prop ;
+	}
+
+	// Look for alias in remote buses
+	bus = RemoteAlias(pn) ;
+	if ( bus == INDEX_BAD ) {
+		return parse_error ;
+	}
+	
+	// Found the alias (remotely)
+	SetKnownBus(bus, pn);
+
+	if ( pn->sn[0] == 0 && pn->sn[7]==0 ) { // no serial number owserver (older)
+		Cache_Add_Alias_Bus(filename,bus) ;
+	} else {
+		Cache_Add_Alias( filename, pn->sn ) ;
+		Cache_Add_Device( bus, pn->sn ) ;
+		pn->selected_device = FS_devicefindhex(pn->sn[0], pn);
+	}
+
+	return parse_prop ;
+}
+
 /* Parse Name (only device name) part of string */
 /* Return -ENOENT if not a valid name
    return 0 if good
@@ -462,16 +571,22 @@ static enum parse_enum Parse_Bus(char *pathnow, struct parsedname *pn)
  */
 static enum parse_enum Parse_RealDevice(char *filename, enum parse_pass remote_status, struct parsedname *pn)
 {
-	int bus_nr ;
-	if ( Parse_SerialNumber(filename,pn->sn)  ) {
-		// Not a serial number, look for an alias
-		if (Cache_Get_SerialNumber(filename,pn->sn)) {
-			return parse_error;
-		}
+	switch ( Parse_SerialNumber(filename,pn->sn) ) {
+		case sn_alias:
+			return Parse_Alias( filename, remote_status, pn) ;
+		case sn_valid:
+			return Parse_RealDeviceSN( remote_status, pn ) ;
+		case sn_invalid:
+		default:
+			return parse_error ;
 	}
+}
 
+/* Device is known with serial number */
+static enum parse_enum Parse_RealDeviceSN(enum parse_pass remote_status, struct parsedname *pn)
+{
 	/* Search for known 1-wire device -- keyed to device name (family code in HEX) */
-	FS_devicefindhex(pn->sn[0], pn);
+	pn->selected_device = FS_devicefindhex(pn->sn[0], pn);
 
 	// returning from owserver -- don't need to check presence (it's implied)
 	if (remote_status == parse_pass_post_remote) {
@@ -479,11 +594,11 @@ static enum parse_enum Parse_RealDevice(char *filename, enum parse_pass remote_s
 	}
 
 	if (Globals.one_device) {
-		bus_nr = 0;				// arbitrary assignment
+		SetKnownBus(INDEX_DEFAULT, pn);
 	} else {
 		/* Check the presence, and cache the proper bus number for better performance */
-		bus_nr = CheckPresence(pn);
-		if (bus_nr < 0) {
+		INDEX_OR_ERROR bus_nr = CheckPresence(pn);
+		if ( INDEX_NOT_VALID(bus_nr) ) {
 			return parse_error;	/* CheckPresence failed */
 		}
 	}
@@ -499,7 +614,7 @@ static enum parse_enum Parse_NonRealDevice(char *filename, struct parsedname *pn
 {
 	//printf("Parse_NonRealDevice: [%s] [%s]\n", filename, pn->path);
 	FS_devicefind(filename, pn);
-	return (pn->selected_device == &NoDevice) ? parse_error : parse_prop;
+	return (pn->selected_device == &UnknownDevice) ? parse_error : parse_prop;
 }
 
 static enum parse_enum Parse_Property(char *filename, struct parsedname *pn)
@@ -508,21 +623,27 @@ static enum parse_enum Parse_Property(char *filename, struct parsedname *pn)
 
 	//printf("FilePart: %s %s\n", filename, pn->path);
 
+	// Special case for remote device. Use distant data
+	if ( pn->selected_device == &RemoteDevice ) {
+		// remote device, no known sn, can't handle a prolerty
+		return parse_error ;
+	}
+
 	filename = strsep(&dot, ".");
 	//printf("FP name=%s, dot=%s\n", filename, dot);
 	/* Match to known filetypes for this device */
 	if ((pn->selected_filetype =
 		 bsearch(filename, pn->selected_device->filetype_array,
-				 (size_t) pn->selected_device->count_of_filetypes, sizeof(struct filetype), filecmp))) {
+				 (size_t) pn->selected_device->count_of_filetypes, sizeof(struct filetype), filetype_cmp))) {
 		//printf("FP known filetype %s\n",pn->selected_filetype->name) ;
 		/* Filetype found, now process extension */
 		if (dot == NULL || dot[0] == '\0') {	/* no extension */
-			if (pn->selected_filetype->ag) {
+			if (pn->selected_filetype->ag != NON_AGGREGATE) {
 				return parse_error;	/* aggregate filetypes need an extension */
 			}
 			pn->extension = 0;	/* default when no aggregate */
 
-		} else if (pn->selected_filetype->ag == NULL) {
+		} else if (pn->selected_filetype->ag == NON_AGGREGATE) {
 			return parse_error;	/* An extension not allowed when non-aggregate */
 
 		} else if (strcasecmp(dot, "ALL") == 0) {
@@ -561,8 +682,12 @@ static enum parse_enum Parse_Property(char *filename, struct parsedname *pn)
 		//printf("FP Good\n") ;
 		switch (pn->selected_filetype->format) {
 		case ft_directory:		// aux or main
-			if (BranchAdd(pn)) {
-				//printf("PN BranchAdd failed for %s\n", pn->path);
+			if ( pn->type == ePN_structure ) {
+				// special case, structure for aux and main
+				return parse_done;
+			}
+			if (BranchAdd(pn) != 0) {
+				//printf("PN BranchAdd failed for %s\n", filename);
 				return parse_error;
 			}
 			/* STATISTICS */
@@ -573,8 +698,9 @@ static enum parse_enum Parse_Property(char *filename, struct parsedname *pn)
 			STATUNLOCK;
 			return parse_branch;
 		case ft_subdir:
+			//printf("PN %s is a subdirectory\n", filename);
 			pn->subdir = pn->selected_filetype;
-			pn->selected_filetype = NULL;
+			pn->selected_filetype = NO_FILETYPE;
 			return parse_subprop;
 		default:
 			return parse_done;
@@ -584,67 +710,67 @@ static enum parse_enum Parse_Property(char *filename, struct parsedname *pn)
 	return parse_error;			/* filetype not found */
 }
 
-static int BranchAdd(struct parsedname *pn)
+static ZERO_OR_ERROR BranchAdd(struct parsedname *pn)
 {
 	//printf("BRANCHADD\n");
 	if ((pn->pathlength % BRANCH_INCR) == 0) {
 		void *temp = pn->bp;
 		if ((pn->bp = owrealloc(temp, (BRANCH_INCR + pn->pathlength) * sizeof(struct buspath))) == NULL) {
-			if (temp) {
-				owfree(temp);
-			}
+			SAFEFREE(temp) ;
 			return -ENOMEM;
 		}
 	}
-	memcpy(pn->bp[pn->pathlength].sn, pn->sn, 8);	/* copy over DS2409 name */
+	memcpy(pn->bp[pn->pathlength].sn, pn->sn, SERIAL_NUMBER_SIZE);	/* copy over DS2409 name */
 	pn->bp[pn->pathlength].branch = pn->selected_filetype->data.i;
 	++pn->pathlength;
-	pn->selected_filetype = NULL;
-	pn->selected_device = NULL;
+	pn->selected_filetype = NO_FILETYPE;
+	pn->selected_device = NO_DEVICE;
 	return 0;
 }
 
-int filecmp(const void *name, const void *ex)
+int filetype_cmp(const void *name, const void *ex)
 {
 	return strcmp((const char *) name, ((const struct filetype *) ex)->name);
 }
 
 /* Parse a path/file combination */
-int FS_ParsedNamePlus(const char *path, const char *file, struct parsedname *pn)
+ZERO_OR_ERROR FS_ParsedNamePlus(const char *path, const char *file, struct parsedname *pn)
 {
-	if (path == NULL || path[0] == '\0') {
-		return FS_ParsedName(file, pn);
-	} else if (file == NULL || file[0] == '\0') {
-		return FS_ParsedName(path, pn);
-	} else {
-		int ret = 0;
-		char *fullpath;
-		fullpath = owmalloc(strlen(file) + strlen(path) + 2);
-		if (fullpath == NULL) {
-			return -ENOMEM;
-		}
-		strcpy(fullpath, path);
-		if (fullpath[strlen(fullpath) - 1] != '/') {
-			strcat(fullpath, "/");
-		}
-		strcat(fullpath, file);
-		//printf("PARSENAMEPLUS path=%s pre\n",fullpath) ;
-		ret = FS_ParsedName(fullpath, pn);
-		//printf("PARSENAMEPLUS path=%s post\n",fullpath) ;
-		owfree(fullpath);
-		//printf("PARSENAMEPLUS free\n") ;
-		return ret;
+	ZERO_OR_ERROR ret = 0;
+	char *fullpath;
+
+	if (path == NO_PATH) {
+		path = "" ;
 	}
+	if (file == NULL) {
+		file = "" ;
+	}
+
+	fullpath = owmalloc(strlen(file) + strlen(path) + 2);
+	if (fullpath == NO_PATH) {
+		return -ENOMEM;
+	}
+	strcpy(fullpath, path);
+	if (fullpath[strlen(fullpath) - 1] != '/') {
+		strcat(fullpath, "/");
+	}
+	strcat(fullpath, file);
+	//printf("PARSENAMEPLUS path=%s pre\n",fullpath) ;
+	ret = FS_ParsedName(fullpath, pn);
+	//printf("PARSENAMEPLUS path=%s post\n",fullpath) ;
+	owfree(fullpath);
+	//printf("PARSENAMEPLUS free\n") ;
+	return ret;
 }
 
 /* Parse a path/file combination */
-int FS_ParsedNamePlusExt(const char *path, const char *file, int extension, enum ag_index alphanumeric, struct parsedname *pn)
+ZERO_OR_ERROR FS_ParsedNamePlusExt(const char *path, const char *file, int extension, enum ag_index alphanumeric, struct parsedname *pn)
 {
 	char name[OW_FULLNAME_MAX];
 	UCLIBCLOCK;
-	if (extension == -2) {
+	if (extension == EXTENSION_BYTE ) {
 		snprintf(name, OW_FULLNAME_MAX, "%s.BYTE", file);
-	} else if (extension == -1) {
+	} else if (extension == EXTENSION_ALL ) {
 		snprintf(name, OW_FULLNAME_MAX, "%s.ALL", file);
 	} else if (alphanumeric == ag_letters) {
 		snprintf(name, OW_FULLNAME_MAX, "%s.%c", file, extension + 'A');
@@ -655,16 +781,7 @@ int FS_ParsedNamePlusExt(const char *path, const char *file, int extension, enum
 	return FS_ParsedNamePlus(path, name, pn);
 }
 
-/* For read and write sibling -- much simpler requirements */
-int FS_ParseProperty_for_sibling(char *filename, struct parsedname *pn)
+void FS_ParsedName_Placeholder( struct parsedname * pn )
 {
-	// need to make a copy of filename since a static string can't be modified by strsep
-	char *filename_copy = owstrdup(filename);
-	int ret;
-	if (filename_copy == NULL) {
-		return 1;
-	}
-	ret = (Parse_Property(filename_copy, pn) == parse_done) ? 0 : 1;
-	owfree(filename_copy);
-	return ret;
+	FS_ParsedName( NULL, pn ) ; // minimal parsename -- no destroy needed
 }
