@@ -1,5 +1,5 @@
 /*
-$Id: ow_stateinfo.c,v 1.5 2008/06/18 15:05:47 d1mag Exp $
+$Id: ow_stateinfo.c,v 1.6 2010/05/25 20:57:38 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
@@ -17,5 +17,5 @@ struct stateinfo StateInfo = {
 	.owlib_state = lib_state_pre,
 	.start_time = 0,
 	.dir_time = 0,
-	.shutdown_in_progress = 0,
+	.shutting_down = 0,
 };
