@@ -1,5 +1,5 @@
 /*
-$Id: ow_1820.h,v 1.7 2007/02/05 05:23:43 alfille Exp $
+$Id: ow_1820.h,v 1.8 2009/12/12 02:10:16 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
@@ -29,7 +29,7 @@ $Id: ow_1820.h,v 1.7 2007/02/05 05:23:43 alfille Exp $
 #ifndef OWFS_CONFIG_H
 #error Please make sure owfs_config.h is included *before* this header file
 #endif
-#include "ow_xxxx.h"
+#include "ow_standard.h"
 
 /* -------- Structures ---------- */
 DeviceHeader(DS18S20);
