@@ -1,5 +1,5 @@
 /*
-$Id: ow_1977.h,v 1.4 2007/01/30 05:19:34 alfille Exp $
+$Id: ow_1977.h,v 1.5 2009/12/12 02:10:16 alfille Exp $
     OWFS -- One-Wire filesystem
     OWHTTPD -- One-Wire Web Server
     Written 2003 Paul H Alfille
@@ -15,7 +15,7 @@ $Id: ow_1977.h,v 1.4 2007/01/30 05:19:34 alfille Exp $
 #ifndef OWFS_CONFIG_H
 #error Please make sure owfs_config.h is included *before* this header file
 #endif
-#include "ow_xxxx.h"
+#include "ow_standard.h"
 
 /* ------- Structures ----------- */
 
