@@ -1,5 +1,5 @@
 /*
-$Id: ow_charblob.h,v 1.4 2009/03/23 01:03:18 alfille Exp $
+$Id: ow_charblob.h,v 1.6 2010/09/24 03:23:11 alfille Exp $
     OW -- One-Wire filesystem
     version 0.4 7/2/2003
 
@@ -26,6 +26,8 @@ $Id: ow_charblob.h,v 1.4 2009/03/23 01:03:18 alfille Exp $
 #ifndef OW_CHARBLOB_H			/* tedious wrapper */
 #define OW_CHARBLOB_H
 
+#define NO_CHARBLOB	NULL
+
 struct charblob {
 	int troubled;
 	size_t allocated;
@@ -38,5 +40,7 @@ void CharblobInit(struct charblob *cb);
 int CharblobPure(struct charblob *cb);
 int CharblobAdd(const ASCII * a, size_t s, struct charblob *cb);
 int CharblobAddChar(const ASCII a, struct charblob *cb);
+ASCII * CharblobData(struct charblob * cb);
+size_t CharblobLength( struct charblob * cb );
 
 #endif							/* OW_CHARBLOB_H */
