@@ -1,4 +1,4 @@
-/* $Id: ow_codes.h,v 1.4 2008/01/30 13:15:17 d1mag Exp $
+/* $Id: ow_codes.h,v 1.6 2010/04/10 16:38:35 alfille Exp $
    Debug and error messages
    Meant to be included in ow.h
    Separated just for readability
@@ -15,6 +15,7 @@
 
 /* 1-wire ROM commands */
 #define _1W_READ_ROM               0x33
+#define _1W_OLD_READ_ROM           0x0F
 #define _1W_MATCH_ROM              0x55
 #define _1W_SKIP_ROM               0xCC
 #define _1W_SEARCH_ROM             0xF0
@@ -23,5 +24,6 @@
 #define _1W_OVERDRIVE_SKIP_ROM     0x3C
 #define _1W_OVERDRIVE_MATCH_ROM    0x69
 #define _1W_SWAP                   0xAA	// used by the DS27xx
+#define _1W_LOCATOR                0x00 // used by the Link Locator
 
 #endif							/* OW_CODES_H */
