@@ -1,6 +1,6 @@
 <?php
 
-$extensionFile = "/usr/lib/php/modules/libowphp.so";
+$extensionFile = "/usr/lib/php5/20090626/libowphp.so";
 
 //make sure that we are ABLE to load libraries
 if( !(bool)ini_get( "enable_dl" ) || (bool)ini_get( "safe_mode" ) ) 
